#include <visa.h>

/*= Useful Macros ===========================================================*/

/* - Basic Macros ---------------------------------------------------------- */

/* Error checking macro */
#define CHECKERR(fCal) if (rssma_status = (fCal), rssma_status < VI_SUCCESS) \
    return rssma_status

/* Error checking macro with jump to the ERROR: label */
#define GOCHECKERR(fCal) if (rssma_status = (fCal), rssma_status < VI_SUCCESS) \
                            goto Error

/* Initialize default function parameters */
#define INIT\
    ViStatus            rssma_status = VI_SUCCESS; \
    rssma_instrRange    instrPtr = NULL; \
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));\
    if (instrPtr == NULL)\
        return VI_ERROR_INSTR_PARAMETER

/* Close function and return */
#define RETURN\
    return rssma_status;

/* - Additional Macros ----------------------------------------------------- */

#define CHECKOPT(option) if (instrPtr->optionChecking == VI_TRUE)\
                         {\
                            if (instrPtr->instrOptions.option == 0)\
                                return RSSMA_ERROR_INSTRUMENT_OPTION;\
                         }

#define CHECKOPTCONT(option) if (instrPtr->optionChecking == VI_TRUE)\
                         {\
                            if (instrPtr->instrOptions.option == 0)\
                                rssma_status = RSSMA_ERROR_INSTRUMENT_OPTION;\
                         }
/* Check NULL pointer */
#define CHECKNULL(pointer) if (pointer == NULL) return rssma_ParameterError(instrSession,#pointer)


/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rssma_stringValPair is used in the rssma_error_message function           */
/* rssma_statusDataRanges is used to track session dependent status & ranges */
/*===========================================================================*/
typedef struct  rssma_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rssma_tStringValPair;

typedef struct rssma_instrOptions
{
    int B20:1;
    int B22:1;
    int B29:1;
    int B80:1;
    int B81:1;
    int B103:1;
    int B106:1;
    int K23:1;
    int K24:1;
    int K25:1;
    int K83:1;
	int K27:1;
	int K26:1;
	int K20:1;
	int K22:1;
	int K28:1;
} rssma_tinstrOptions;

struct rssma_statusDataRanges
{
    ViChar              instrDriverRevision[256];
    ViBoolean           errorChecking;
    ViBoolean           optionChecking;
    ViBoolean           rangeChecking;
    rssma_tinstrOptions instrOptions;
	ViChar      parameterErrorMessage[256];
};

typedef struct rssma_statusDataRanges *rssma_instrRange;
ViBoolean rssma_invalidViBooleanRange (ViSession instrSession, ViBoolean val);
ViBoolean rssma_invalidViInt32Range (ViSession instrSession, ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rssma_invalidViReal64Range (ViSession instrSession, ViReal64 val, ViReal64 min,
                                       ViReal64 max);
ViStatus rssma_checkStatus (ViSession instrSession);
ViStatus rssma_waitOPC (ViSession instrSession, ViPString command,
                        ViPString instrResp, ViInt32 timeout);
ViStatus rssma_sendCmd (ViSession instrSession, ViString cmd);
ViStatus rssma_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViString parameterName);
ViStatus rssma_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViString parameterName);
ViStatus rssma_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViString parameterName);
ViStatus rssma_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViString parameterName);
ViStatus rssma_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViString parameterName);
ViStatus rssma_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViString parameterName);
ViStatus rssma_setViReal64Units (ViSession instrSession,
                                 ViString cmd,
                                 ViReal64 value,
                                 ViString units,
                                 ViReal64 min,
                                 ViReal64 max,
                                 ViString parameterName);
ViStatus rssma_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViString parameterName);
ViStatus rssma_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViString parameterName);
ViStatus rssma_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViString parameterName);
ViStatus rssma_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViString parameterName);
ViStatus rssma_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViString parameterName);
ViStatus rssma_checkIndexInList (ViSession instrSession,
								 ViInt32 index,
                                 ViString list[],
                                 ViString parameterName);
ViStatus rssma_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index);
ViStatus rssma_getCatalog (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViString parameterName);
ViStatus rssma_getCatalogWithSeparator (ViSession instrSession,
                           ViString cmd,
						   ViChar separator,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViString parameterName);
ViStatus rssma_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value[],
                            ViInt32  count,
                            ViString parameterName);
ViStatus rssma_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value[],
                            ViInt32  count,
                            ViString parameterName);
ViStatus rssma_getViInt32Array (ViSession instrSession,
                            ViString cmd,
                            ViInt32 value[],
                            ViInt32  count,
                            ViString parameterName);
ViStatus rssma_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count);
ViStatus rssma_readDataToFile (ViSession instrSession, ViString cmd,
                           ViString destination);

ViStatus rssma_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViString parameterName);
ViStatus rssma_ParameterError (ViSession io, ViString parameterName);
