/*= R&S SMA Signal Generator Include File ===================================*/
/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/

#ifndef __RSSMA_HEADER
#define __RSSMA_HEADER

#include <visa.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_WARN_NSUP_ID_QUERY     (          0x3FFC0101L)
#define VI_WARN_NSUP_RESET        (          0x3FFC0102L)
#define VI_WARN_NSUP_SELF_TEST    (          0x3FFC0103L)
#define VI_WARN_NSUP_ERROR_QUERY  (          0x3FFC0104L)
#define VI_WARN_NSUP_REV_QUERY    (          0x3FFC0105L)

#define VI_ERROR_FAIL_ID_QUERY    (_VI_ERROR+0x3FFC0011L)
#define VI_ERROR_INV_RESPONSE     (_VI_ERROR+0x3FFC0012L)

#define VI_ERROR_INSTR_FILE_OPEN                    (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE                   (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE        (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER                    (_VI_ERROR+0x3FFC0810L)

/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                          (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                       (_VI_ERROR+0x3FFC0900L)

#define RSSMA_ERROR_INSTRUMENT_ERROR                (VI_ERROR_INSTR_OFFSET + 0xF0L)
/* Add instrument driver specific error codes here */

#define RSSMA_ERROR_INVALID_CONFIGURATION           (VI_ERROR_INSTR_OFFSET + 0xF1L)
#define RSSMA_ERROR_INSTRUMENT_OPTION               (VI_ERROR_INSTR_OFFSET + 0xF2L)
#define RSSMA_ERROR_INSTRUMENT_NSUP_MODEL           (VI_ERROR_INSTR_OFFSET + 0xF3L)
#define RSSMA_ERROR_SETTINGS_CONFLICT               (VI_ERROR_INSTR_OFFSET + 0xF4L)
#define RSSMA_ERROR_CALIBRATION_FAILED              (VI_ERROR_INSTR_OFFSET + 0xF5L)
/* Add instrument driver specific warning codes here */
#define RSSMA_WARNING_SMALL_BUFFER                  (VI_WARNING_INSTR_OFFSET + 0xF1L)


/*****************************************************************************/
/*= Instrument specific defines =============================================*/
/*****************************************************************************/

/* --- Last channel ----------------------------------------------------------- */
#define RSSMA_CHANNEL_NUM                                   RSSMA_CHANNEL_1

/* --- Channel ---------------------------------------------------------------- */
                                      
#define RSSMA_CHANNEL_1                                     1
#define RSSMA_CHANNEL_2                                     2

/* --- Set RF Frequency on Recall ------------------------------------------ */

#define RSSMA_FREQUENCY_RCL_INCLUDE                         0
#define RSSMA_FREQUENCY_RCL_EXCLUDE                         1

/* --- Set RF Level on Recall ---------------------------------------------- */

#define RSSMA_POWER_LEVEL_RCL_INCLUDE                       0
#define RSSMA_POWER_LEVEL_RCL_EXCLUDE                       1

/* --- Set ROSC Source ----------------------------------------------------- */

#define RSSMA_ROSC_SOURCE_INTERNAL                          0
#define RSSMA_ROSC_SOURCE_EXTERNAL                          1

/* --- Set ROSC External Reference Frequency ------------------------------- */

#define RSSMA_EXTERN_REF_FREQ_5MHZ                          0
#define RSSMA_EXTERN_REF_FREQ_10MHZ                         1
#define RSSMA_EXTERN_REF_FREQ_13MHZ                         2

/* --- Set ROSC Synchronization Bandwidth ---------------------------------- */

#define RSSMA_ROSC_SBAN_STANDARD                            0
#define RSSMA_ROSC_SBAN_NARROW                              1

/* --- Set Attenuator Mode ------------------------------------------------- */

#define RSSMA_OUTPUT_AMODE_AUTO                             0
#define RSSMA_OUTPUT_AMODE_FIXED                            1
#define RSSMA_OUTPUT_AMODE_NORM                             2
#define RSSMA_OUTPUT_AMODE_HPOW                             3

/* --- Set Attenuator RF Off Mode ------------------------------------------ */

#define RSSMA_OUTPUT_RFOFF_UNCH								0
#define RSSMA_OUTPUT_RFOFF_FATT								1

/* --- Set Power On State -------------------------------------------------- */

#define RSSMA_OUTPUT_STATE_PON_OFF                          0
#define RSSMA_OUTPUT_STATE_PON_UNCHANGED                    1

/* --- ALC State ----------------------------------------------------------- */

#define RSSMA_POWER_ALC_STATE_OFF                           0
#define RSSMA_POWER_ALC_STATE_ON                            1   
#define RSSMA_POWER_ALC_STATE_AUTO                          2

/* --- Set LFGen Source --------------------------------------------------- */

#define RSSMA_LFOUTPUT_SOURCE_LF1                           0
#define RSSMA_LFOUTPUT_SOURCE_LF2                           1
#define RSSMA_LFOUTPUT_SOURCE_LF12                          2
#define RSSMA_LFOUTPUT_SOURCE_NOISE                         3
#define RSSMA_LFOUTPUT_SOURCE_LF1NOISE                      4
#define RSSMA_LFOUTPUT_SOURCE_LF2NOISE                      5

/* --- Set LF Sweep Spacing ----------------------------------------------- */
#define RSSMA_LFOUTPUT_SWEEP_FREQUENCY_SPACING_LINEAR       0
#define RSSMA_LFOUTPUT_SWEEP_FREQUENCY_SPACING_LOGARITHMIC  1

/* --- Set CSYN Power Output Mode ----------------------------------------- */

#define RSSMA_CSYN_POWER_MODE_ECL                           0
#define RSSMA_CSYN_POWER_MODE_CMOS                          1

/* --- Set AM Source ------------------------------------------------------ */

#define RSSMA_AM_SOURCE_INT                                 0
#define RSSMA_AM_SOURCE_EXT                                 1
#define RSSMA_AM_SOURCE_INTEXT                              2

/* --- Set AM External Coupling ------------------------------------------- */

#define RSSMA_AM_EXTERNAL_COUPLING_AC                       0
#define RSSMA_AM_EXTERNAL_COUPLING_DC                       1

/* --- Set AM Internal Source --------------------------------------------- */

#define RSSMA_AM_INTERNAL_SOURCE_LF1                        0
#define RSSMA_AM_INTERNAL_SOURCE_LF2                        1
#define RSSMA_AM_INTERNAL_SOURCE_LF12                       2
#define RSSMA_AM_INTERNAL_SOURCE_NOISE                      3
#define RSSMA_AM_INTERNAL_SOURCE_LF1NOISE                   4
#define RSSMA_AM_INTERNAL_SOURCE_LF2NOISE                   5

/* --- Set PULM Polarity -------------------------------------------------- */

#define RSSMA_PULM_POLARITY_NORMAL                          0
#define RSSMA_PULM_POLARITY_INVERTED                        1

/* --- Set PULM Source ---------------------------------------------------- */

#define RSSMA_PULM_SOURCE_INTERNAL                          0
#define RSSMA_PULM_SOURCE_EXTERNAL                          1
#define RSSMA_PULM_SOURCE_CODER                             2

/* --- Set PULM Trigger Mode ---------------------------------------------- */

#define RSSMA_PULM_TRIGGER_MODE_AUTO                        0
#define RSSMA_PULM_TRIGGER_MODE_EXTERNAL                    1
#define RSSMA_PULM_TRIGGER_MODE_EGATE                       2

/* --- Set FM Source ------------------------------------------------------ */

#define RSSMA_FM_SOURCE_INTERNAL                            0
#define RSSMA_FM_SOURCE_EXTERNAL                            1
#define RSSMA_FM_SOURCE_INTEXT                              2
#define RSSMA_FM_SOURCE_EDIG                                3

/* --- Set FM Mode -------------------------------------------------------- */

#define RSSMA_FM_MODE_NORMAL                                0
#define RSSMA_FM_MODE_LNOIS                                 1

/* --- Set PM Mode -------------------------------------------------------- */

#define RSSMA_PM_MODE_HBAN                                  0
#define RSSMA_PM_MODE_HDEV                                  1
#define RSSMA_PM_MODE_LNOIS                                 2

/* --- Set FM External Coupling ------------------------------------------- */

#define RSSMA_EXTERNAL_COUPLING_AC                          0
#define RSSMA_EXTERNAL_COUPLING_DC                          1

/* --- Set FM Internal Source --------------------------------------------- */

#define RSSMA_FM_INTERNAL_SOURCE_LF1                        0
#define RSSMA_FM_INTERNAL_SOURCE_LF2                        1
#define RSSMA_FM_INTERNAL_SOURCE_LF12                       2
#define RSSMA_FM_INTERNAL_SOURCE_NOISE                      3
#define RSSMA_FM_INTERNAL_SOURCE_LF1NOISE                   4
#define RSSMA_FM_INTERNAL_SOURCE_LF2NOISE                   5

/* --- Set FM External Binary Format -------------------------------------- */

#define RSSMA_FM_EXTERNAL_DIGITAL_FORMAT_BOFFSET            0
#define RSSMA_FM_EXTERNAL_DIGITAL_FORMAT_DCODE              1

/* --- Set FM External Digital Mode --------------------------------------- */

#define RSSMA_FM_EXTERNAL_DIGITAL_MODE_AUTO                 0
#define RSSMA_FM_EXTERNAL_DIGITAL_MODE_LNOIS                1
#define RSSMA_FM_EXTERNAL_DIGITAL_MODE_WIDE                 2

/* --- Set NOISE Distribution --------------------------------------------- */

#define RSSMA_NOISE_DISTRIBUTION_GAUSS                      0
#define RSSMA_NOISE_DISTRIBUTION_EQUAL                      1

/* --- Set Instrument Trigger Input Slope --------------------------------- */

#define RSSMA_INPUT_TRIGGER_SLOPE_NEGATIVE                  0
#define RSSMA_INPUT_TRIGGER_SLOPE_POSITIVE                  1

/* --- Set Instrument Trigger Input Slope --------------------------------- */

#define RSSMA_EXT_INPUT_IMPEDANCE_G50                       0
#define RSSMA_EXT_INPUT_IMPEDANCE_G1K                       1

/* --- Frequency Sweep Mode ------------------------------------------------ */

#define RSSMA_SWEEP_MODE_AUTO                               0
#define RSSMA_SWEEP_MODE_SINGLE                             1
#define RSSMA_SWEEP_MODE_STEP                               2
#define RSSMA_SWEEP_MODE_EXT_SINGLE                         3
#define RSSMA_SWEEP_MODE_EXT_STEP                           4
#define RSSMA_SWEEP_MODE_EXT_START                          5
#define RSSMA_SWEEP_MODE_EXT_HOP                            5
#define RSSMA_SWEEP_MODE_EXT_DHOP                           6

#define RSSMA_LIST_SWEEP_MODE_AUTO                          0
#define RSSMA_LIST_SWEEP_MODE_STEP                          1

    
#define RSSMA_SWEEP_LOW_MODE_AUTO                           0
#define RSSMA_SWEEP_LOW_MODE_MAN                            1
#define RSSMA_SWEEP_LOW_MODE_STEP                           2

#define RSSMA_FREQ_CENTER_SPAN                              0
#define RSSMA_FREQ_START_STOP                               1

#define RSSMA_FREQ_SWEEP_MODE_AUTO                          0
#define RSSMA_FREQ_SWEEP_MODE_SINGLE                        1
#define RSSMA_FREQ_SWEEP_MODE_STEP                          2
#define RSSMA_FREQ_SWEEP_MODE_EXT_SINGLE                    3
#define RSSMA_FREQ_SWEEP_MODE_EXT_STEP                      4

/* --- TRIGger SOURce All Sweeps ------------------------------------------- */

#define RSSMA_TRIG_SOURCE_ALL_SWEEPS_AUTO                   0
#define RSSMA_TRIG_SOURCE_ALL_SWEEPS_IMMEDIATE              1
#define RSSMA_TRIG_SOURCE_ALL_SWEEPS_SINGLE                 2
#define RSSMA_TRIG_SOURCE_ALL_SWEEPS_EXTERNAL               3
#define RSSMA_TRIG_SOURCE_ALL_SWEEPS_BUS                    4

#define RSSMA_TRIG_SOURCE_SWEEP_AUTO                        0
#define RSSMA_TRIG_SOURCE_SWEEP_IMMEDIATE                   1
#define RSSMA_TRIG_SOURCE_SWEEP_SINGLE                      2
#define RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL                    3
#define RSSMA_TRIG_SOURCE_SWEEP_BUS                         4
#define RSSMA_TRIG_SOURCE_SWEEP_EAUTO	                    5

/* --- FREQuency Mode ------------------------------------------------------ */

#define RSSMA_FREQUENCY_MODE_CW                             0
#define RSSMA_FREQUENCY_MODE_SWEEP                          1
#define RSSMA_FREQUENCY_MODE_LIST                           2

/* --- Frequency Sweep Spacing --------------------------------------------- */

#define RSSMA_FREQ_SWEEP_SPACING_LIN                        0
#define RSSMA_FREQ_SWEEP_SPACING_LOG                        1

/* --- POWer RF Level Mode ------------------------------------------------- */

#define RSSMA_POWER_MODE_CW                                 0
#define RSSMA_POWER_MODE_SWEEP                              1
#define RSSMA_POWER_MODE_LIST                               2

/* --- LFOutput Mode ------------------------------------------------------- */

#define RSSMA_LFOUTPUT_FREQUENCY_MODE_CW                    0
#define RSSMA_LFOUTPUT_FREQUENCY_MODE_SWEEP                 1

/* --- TRIGger SOURce RF Level Sweep --------------------------------------- */

#define RSSMA_TRIG_SOURCE_RF_LEVEL_SWEEP_AUTO               0
#define RSSMA_TRIG_SOURCE_RF_LEVEL_SWEEP_IMMEDIATE          1
#define RSSMA_TRIG_SOURCE_RF_LEVEL_SWEEP_SINGLE             2
#define RSSMA_TRIG_SOURCE_RF_LEVEL_SWEEP_EXTERNAL           3
#define RSSMA_TRIG_SOURCE_RF_LEVEL_SWEEP_BUS                4

/* --- LIST Trigger Source ------------------------------------------------- */

#define RSSMA_LIST_TRIGGER_SOURCE_AUTO                      0
#define RSSMA_LIST_TRIGGER_SOURCE_SINGLE                    1
#define RSSMA_LIST_TRIGGER_SOURCE_EXTERNAL                  2
#define RSSMA_LIST_TRIGGER_SOURCE_HOP                       3
#define RSSMA_LIST_TRIGGER_SOURCE_DHOP                      4
/* --- Frequency Sweep Mode ------------------------------------------------ */

#define RSSMA_LEVEL_SWEEP_MODE_AUTO                         0
#define RSSMA_LEVEL_SWEEP_MODE_MANUAL                       1
#define RSSMA_LEVEL_SWEEP_MODE_STEP                         2

/* --- Registers ----------------------------------------------------------- */
#define RSSMA_REGISTER_OPERATION                            0
#define RSSMA_REGISTER_QUESTIONABLE                         1
#define RSSMA_REGISTER_COND                                 0
#define RSSMA_REGISTER_EVENT                                1
#define RSSMA_REGISTER_ENABLE                               2
#define RSSMA_REGISTER_NTR                                  3
#define RSSMA_REGISTER_PTR                                  4
/* --- Set LFGen Shape ----------------------------------------------------- */
#define RSSMA_LFOUTPUT_SHAPE_SINE                           0
#define RSSMA_LFOUTPUT_SHAPE_SQUARE                         1
#define RSSMA_LFOUTPUT_SHAPE_TRIANGLE                       2
#define RSSMA_LFOUTPUT_SHAPE_TRAPEZE                        3
    
/* --- Set LFGen Shape Trapeze --------------------------------------------- */

#define RSSMA_LFOUTPUT_SHAPE_TRAPEZE_FALL                   0
#define RSSMA_LFOUTPUT_SHAPE_TRAPEZE_HIGH                   1
#define RSSMA_LFOUTPUT_SHAPE_TRAPEZE_LOW                    2
#define RSSMA_LFOUTPUT_SHAPE_TRAPEZE_RISE                   3

/* --- TEST ---------------------------------------------------------------- */

#define RSSMA_TEST_COMMAND_SSYN                             0
#define RSSMA_TEST_COMMAND_SSYNEX                           1
#define RSSMA_TEST_COMMAND_IQOP3                            2
#define RSSMA_TEST_COMMAND_IQOP6                            3
#define RSSMA_TEST_COMMAND_SATT3                            4
#define RSSMA_TEST_COMMAND_SATT6                            5
#define RSSMA_TEST_COMMAND_BBINS                            6
#define RSSMA_TEST_COMMAND_DIRECTLAST                       7
    
/* --- VOR Mode ------------------------------------------------------------ */

#define RSSMA_VORM_MODE_NORM                                0
#define RSSMA_VORM_MODE_VAR                                 1
#define RSSMA_VORM_MODE_SUBC                                2
#define RSSMA_VORM_MODE_FMS                                 3
    
/* --- VOR Direction ------------------------------------------------------- */

#define RSSMA_VORM_DIR_FROM                                 0
#define RSSMA_VORM_DIR_TO                                   1
    
/* --- EXT AM -------------------------------------------------------------- */

#define RSSMA_VORM_SRC_INT                                  0
#define RSSMA_VORM_SRC_EXT                                  1

/* --- VOR Carrier Frequency Step ------------------------------------------ */

#define RSSMA_VAL_VORM_CARR_FREQ_STEP_DECIMAL				0
#define RSSMA_VAL_VORM_CARR_FREQ_STEP_ICAO					1
    
/* --- ILS GS Mode --------------------------------------------------------- */

#define RSSMA_ILSGSM_MODE_NORM                              0
#define RSSMA_ILSGSM_MODE_ULOB                              1
#define RSSMA_ILSGSM_MODE_LLOB                              2
    
/* --- ILS GS Fly Direction ------------------------------------------------ */

#define RSSMA_ILSGSM_DDM_DIR_UP                             0
#define RSSMA_ILSGSM_DDM_DIR_DOWN                           1
    
/* --- EXT AM -------------------------------------------------------------- */

#define RSSMA_ILSGSM_SRC_INT                                0
#define RSSMA_ILSGSM_SRC_EXT                                1
    
/* --- ILS LOC Mode -------------------------------------------------------- */

#define RSSMA_ILSLOCM_MODE_NORM                             0
#define RSSMA_ILSLOCM_MODE_LLOB                             1
#define RSSMA_ILSLOCM_MODE_RLOB                             2
    
/* --- ILS LOC Fly Direction ----------------------------------------------- */

#define RSSMA_ILSLOCM_DDM_DIR_LEFT                          0
#define RSSMA_ILSLOCM_DDM_DIR_RIGHT                         1
    
/* --- Marker Beacon Marker Frequency -------------------------------------- */

#define RSSMA_MBM_MARKER_FREQ_400                           0
#define RSSMA_MBM_MARKER_FREQ_1300                          1
#define RSSMA_MBM_MARKER_FREQ_3000                          2
    
/* --- LIST Import Export List Mode ---------------------------------------- */

#define RSSMA_DEXC_MODE_IMPORT                              0
#define RSSMA_DEXC_MODE_EXPORT                              1
    
/* --- LIST Import Export ASCII Extension ---------------------------------- */

#define RSSMA_DEXC_ASCII_SOURCE_FILE_EXT_TXT                0
#define RSSMA_DEXC_ASCII_SOURCE_FILE_EXT_CSV                1
    
/* --- LIST Import Export ASCII Column Separator --------------------------- */

#define RSSMA_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_TAB         0
#define RSSMA_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_SEM         1
#define RSSMA_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_COMMA       2
#define RSSMA_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_SPACE       3
    
/* --- LIST Import Export ASCII Column Decimal Separator ------------------- */

#define RSSMA_DEXC_ASCII_SOURCE_FILE_COLUMN_DEC_SEP_DOT     0
#define RSSMA_DEXC_ASCII_SOURCE_FILE_COLUMN_DEC_SEP_COMMA   1
    
/* --- Power Unit ---------------------------------------------------------- */

#define RSSMA_SENSE_POWER_UNIT_DBM                          0
#define RSSMA_SENSE_POWER_UNIT_DBUV                         1
#define RSSMA_SENSE_POWER_UNIT_WATT                         2

/* --- Power Source -------------------------------------------------------- */

#define RSSMA_SENSE_POWER_SOURCE_RF                         0
#define RSSMA_SENSE_POWER_SOURCE_A                          0
#define RSSMA_SENSE_POWER_SOURCE_USER                       1
#define RSSMA_SENSE_POWER_SOURCE_B                          2

    
/* --- Power Filter Type --------------------------------------------------- */

#define RSSMA_SENSE_POWER_FILTER_TYPE_AUTO                  0
#define RSSMA_SENSE_POWER_FILTER_TYPE_USER                  1
#define RSSMA_SENSE_POWER_FILTER_TYPE_NSR                   2
    
/* --- Keyboard Layout ----------------------------------------------------- */

#define RSSMA_SYSTEM_KEYBOARD_US                            0
#define RSSMA_SYSTEM_KEYBOARD_DE                            1
    
/* --- Display Blank Mode -------------------------------------------------- */

#define RSSMA_DISPLAY_BLANK_MODE_ON                         0
#define RSSMA_DISPLAY_BLANK_MODE_OFF                        1
#define RSSMA_DISPLAY_BLANK_MODE_AUTO                       2
    
/* --- Display Proportional Scale ------------------------------------------ */

#define RSSMA_DISPLAY_PROPORTIONAL_SCALE_S0V5               0
#define RSSMA_DISPLAY_PROPORTIONAL_SCALE_S1V0               1
    
/* --- SYSTem Identification ----------------------------------------------- */

#define RSSMA_SYSTEM_IDENTIFICATION_HP8662                  0
#define RSSMA_SYSTEM_IDENTIFICATION_HP8663                  1
#define RSSMA_SYSTEM_IDENTIFICATION_ORIGINAL                2

#define RSSMA_SYSTEM_IDENTIFICATION_AUTO                    0
#define RSSMA_SYSTEM_IDENTIFICATION_USER                    1
    
/* --- Network IP Address Mode --------------------------------------------- */

#define RSSMA_SYSTEM_COMMUNICATE_IP_MODE_STATIC             0
#define RSSMA_SYSTEM_COMMUNICATE_IP_MODE_P2P                1
#define RSSMA_SYSTEM_COMMUNICATE_IP_MODE_AUTO               2
    
/* --- Network DNS Mode ---------------------------------------------------- */

#define RSSMA_SYSTEM_COMMUNICATE_DNS_MODE_STATIC            0
#define RSSMA_SYSTEM_COMMUNICATE_DNS_MODE_AUTO              1

/* --- LF Sweep Source ----------------------------------------------------- */

#define RSSMA_LFSWEEP_SOURCE_LF1                            0
#define RSSMA_LFSWEEP_SOURCE_LF2                            1
    
/* --- ILS DDM Coupling ---------------------------------------------------- */

#define RSSMA_ILS_DDM_COUP_FIXED                            0
#define RSSMA_ILS_DDM_COUP_SDM                              1

/* --- Units --------------------------------------------------------------- */
#define RSSMA_UNIT_ANGLE_DEG                                0
#define RSSMA_UNIT_ANGLE_RAD                                1
   
/* --- Velocity ------------------------------------------------------------ */
#define RSSMA_UNIT_VELOCITY_MPS                             0
#define RSSMA_UNIT_VELOCITY_KMH                             1
#define RSSMA_UNIT_VELOCITY_MPH                             2
    
/* --- HCOPY --------------------------------------------------------------- */
    
#define RSSMA_HCOPY_LANG_BMP                                0
#define RSSMA_HCOPY_LANG_JPG                                1
#define RSSMA_HCOPY_LANG_XPM                                2
#define RSSMA_HCOPY_LANG_PNG                                3
    
#define RSSMA_HCOPY_DEV_FILE                                0   
    
#define RSSMA_HCOPY_AUTO_FILE_DAY                           0
#define RSSMA_HCOPY_AUTO_FILE_MONTH                         1
#define RSSMA_HCOPY_AUTO_FILE_YEAR                          2
    
#define RSSMA_HCOPY_SIZE_320_240                            0
#define RSSMA_HCOPY_SIZE_640_480                            1
#define RSSMA_HCOPY_SIZE_800_600                            2
#define RSSMA_HCOPY_SIZE_1024_768                           3   
    
    
/* --- ILS-GSM Carrirer Frequency Step ------------------------------------- */
#define RSSMA_ILSGSM_FREQ_STEP_DEC                          0
#define RSSMA_ILSGSM_FREQ_STEP_ICAO                         1
    
/* --- CHIRP Trigger Level ------------------------------------------------- */
#define RSSMA_CHIRP_TRIGGER_LEVEL_TTL                       0
#define RSSMA_CHIRP_TRIGGER_LEVEL_POV5                      1
#define RSSMA_CHIRP_TRIGGER_LEVEL_M2V5                      2
    
/* --- CHIRP Trigger Mode -------------------------------------------------- */ 
#define RSSMA_CHIRP_TRIGGER_MODE_AUTO                       0
#define RSSMA_CHIRP_TRIGGER_MODE_EXT                        1
#define RSSMA_CHIRP_TRIGGER_MODE_EGAT                       2
    
/* --- Frequency Sweep Shape ----------------------------------------------- */
#define RSSMA_FREQ_SWEEP_SHAPE_SAWT                         0
#define RSSMA_FREQ_SWEEP_SHAPE_TRI                          1
    
/* --- Phase Continuous Frequency Range ------------------------------------ */
#define RSSMA_PHASE_CONT_FREQ_RANGE_NARR                    0
#define RSSMA_PHASE_CONT_FREQ_RANGE_WIDE                    1

/* --- ICAO Channel -------------------------------------------------------- */
#define RSSMA_SOURCE_ICAO_CHANNEL_X                    0 
#define RSSMA_SOURCE_ICAO_CHANNEL_Y                    1 

/* --- ILS ICAO Channel ---------------------------------------------------- */ 
#define RSSMA_ILSGSM_ICAO_CHAN_18_X                         0
#define RSSMA_ILSGSM_ICAO_CHAN_18_Y                         1
#define RSSMA_ILSGSM_ICAO_CHAN_20_X                         2
#define RSSMA_ILSGSM_ICAO_CHAN_20_Y                         3
#define RSSMA_ILSGSM_ICAO_CHAN_22_X                         4
#define RSSMA_ILSGSM_ICAO_CHAN_22_Y                         5
#define RSSMA_ILSGSM_ICAO_CHAN_24_X                         6
#define RSSMA_ILSGSM_ICAO_CHAN_24_Y                         7
#define RSSMA_ILSGSM_ICAO_CHAN_26_X                         8
#define RSSMA_ILSGSM_ICAO_CHAN_26_Y                         9
#define RSSMA_ILSGSM_ICAO_CHAN_28_X                         10
#define RSSMA_ILSGSM_ICAO_CHAN_28_Y                         11
#define RSSMA_ILSGSM_ICAO_CHAN_30_X                         12
#define RSSMA_ILSGSM_ICAO_CHAN_30_Y                         13
#define RSSMA_ILSGSM_ICAO_CHAN_32_X                         14
#define RSSMA_ILSGSM_ICAO_CHAN_32_Y                         15
#define RSSMA_ILSGSM_ICAO_CHAN_34_X                         16
#define RSSMA_ILSGSM_ICAO_CHAN_34_Y                         17
#define RSSMA_ILSGSM_ICAO_CHAN_36_X                         18
#define RSSMA_ILSGSM_ICAO_CHAN_36_Y                         19
#define RSSMA_ILSGSM_ICAO_CHAN_38_X                         20
#define RSSMA_ILSGSM_ICAO_CHAN_38_Y                         21
#define RSSMA_ILSGSM_ICAO_CHAN_40_X                         22
#define RSSMA_ILSGSM_ICAO_CHAN_40_Y                         23
#define RSSMA_ILSGSM_ICAO_CHAN_42_X                         24
#define RSSMA_ILSGSM_ICAO_CHAN_42_Y                         25
#define RSSMA_ILSGSM_ICAO_CHAN_44_X                         26
#define RSSMA_ILSGSM_ICAO_CHAN_44_Y                         27
#define RSSMA_ILSGSM_ICAO_CHAN_46_X                         28
#define RSSMA_ILSGSM_ICAO_CHAN_46_Y                         29
#define RSSMA_ILSGSM_ICAO_CHAN_48_X                         30
#define RSSMA_ILSGSM_ICAO_CHAN_48_Y                         31
#define RSSMA_ILSGSM_ICAO_CHAN_50_X                         32
#define RSSMA_ILSGSM_ICAO_CHAN_50_Y                         33
#define RSSMA_ILSGSM_ICAO_CHAN_52_X                         34
#define RSSMA_ILSGSM_ICAO_CHAN_52_Y                         35
#define RSSMA_ILSGSM_ICAO_CHAN_54_X                         36
#define RSSMA_ILSGSM_ICAO_CHAN_54_Y                         37
#define RSSMA_ILSGSM_ICAO_CHAN_56_X                         38
#define RSSMA_ILSGSM_ICAO_CHAN_56_Y                         39
    
/* --- CHIRP Trigger Slope ------------------------------------------------- */ 
#define RSSMA_CHIRP_TRIGGER_SLOPE_POS                       0
#define RSSMA_CHIRP_TRIGGER_SLOPE_NEG                       1
    
/* --- CHIRP Gate Polarity ------------------------------------------------- */     
#define RSSMA_CHIRP_GATE_POLARITY_INV                       0
#define RSSMA_CHIRP_GATE_POLARITY_NORM                      1
    
/* --- CHIRP Impedance ----------------------------------------------------- */     
#define RSSMA_CHIRP_IMPEDANCE_G10K                          0
#define RSSMA_CHIRP_IMPEDANCE_G50                           1
    
/* --- CHIRP Direction ----------------------------------------------------- */ 
#define RSSMA_CHIRP_DIRECTION_UP                            0
#define RSSMA_CHIRP_DIRECTION_DOWN                          1
    
/* --- DMEM Mode ----------------------------------------------------------- */     
#define RSSMA_DMEM_MODE_INT                                 0
#define RSSMA_DMEM_MODE_REP                                 1
    
/* --- DMEM Pulse Shape ---------------------------------------------------- */ 
#define RSSMA_DMEM_SHAPE_COS                                0
#define RSSMA_DMEM_SHAPE_COS2                               1
#define RSSMA_DMEM_SHAPE_LIN                                2
    
/* --- DMEM Trigger Mode --------------------------------------------------- */ 
#define RSSMA_DME_TRIGGER_MODE_AUTO                         0
#define RSSMA_DME_TRIGGER_MODE_EXTERNAL                     1
#define RSSMA_DME_TRIGGER_MODE_EGATE                        2
#define RSSMA_DME_TRIGGER_MODE_PSEN                         3
    
/* --- DMEM Channel Suffix ------------------------------------------------- */     
#define RSSMA_SOURCE_DMEM_CHANNEL_SUFFIC_X                  0
#define RSSMA_SOURCE_DMEM_CHANNEL_SUFFIC_Y                  1
#define RSSMA_SOURCE_DMEM_CHANNEL_SUFFIC_ICAO				2

/* --- DMEM Composite AM Modulation Component ------------------------------ */
#define RSSMA_SOURCE_DMEM_AM_SIGNAL_BOTH   					0
#define RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN1 					1
#define RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN2 					2

/* --- Power Units --------------------------------------------------------- */     
#define RSSMA_UNIT_POWER_V                                  0
#define RSSMA_UNIT_POWER_DBM                                1
    
/* --- Input Impedance------------------------------------------------------ */     
#define RSSMA_INPUT_IMPEDANCE_G50                           0
#define RSSMA_INPUT_IMPEDANCE_G10K                          1
    
/* --- Pulse Modulation Mode ----------------------------------------------- */     
#define RSSMA_PULM_MODE_SING                                0
#define RSSMA_PULM_MODE_DOUB                                1
#define RSSMA_PULM_MODE_PTR                                 2
    
/* --- ALC Off Mode -------------------------------------------------------- */     
#define RSSMA_ALC_OFF_MODE_SHOLD                            0
    
#define RSSMA_SENSE_POWER_SWEEP_MODE_FREQ                   0
#define RSSMA_SENSE_POWER_SWEEP_MODE_POW                    1
#define RSSMA_SENSE_POWER_SWEEP_MODE_TIME                   2
    
/* --- Power Sensor Time Trigger Source ------------------------------------ */     
#define RSSMA_SENSE_POWER_TIME_TRIG_SOURCE_AUTO             0
#define RSSMA_SENSE_POWER_TIME_TRIG_SOURCE_FREE             1
#define RSSMA_SENSE_POWER_TIME_TRIG_SOURCE_EXT              2
    
/* --- Power Sensor Display Indication ------------------------------------- */     
#define RSSMA_SENSE_POWER_DISP_PULS_ALL                     0
#define RSSMA_SENSE_POWER_DISP_PULS                         1
#define RSSMA_SENSE_POWER_DISP_MARK                         2
#define RSSMA_SENSE_POWER_DISP_GATE                         3
#define RSSMA_SENSE_POWER_DISP_STAN                         4
#define RSSMA_SENSE_POWER_DISP_FULL_SCR                     5
#define RSSMA_SENSE_POWER_DISP_POWER_AVER                   6
#define RSSMA_SENSE_POWER_DISP_POWER_HREF                   7
#define RSSMA_SENSE_POWER_DISP_POWER_REF                    8
#define RSSMA_SENSE_POWER_DISP_POWER_LREF                   9
#define RSSMA_SENSE_POWER_DISP_POWER_MAX                    10
#define RSSMA_SENSE_POWER_DISP_POWER_MIN                    11
#define RSSMA_SENSE_POWER_DISP_POWER_PULSE_BASE             12
#define RSSMA_SENSE_POWER_DISP_POWER_PULSE_TOP              13
#define RSSMA_SENSE_POWER_DISP_PULSE_DCYCLE                 14
#define RSSMA_SENSE_POWER_DISP_PULSE_DUR                    15
#define RSSMA_SENSE_POWER_DISP_PULSE_PERIOD                 16
#define RSSMA_SENSE_POWER_DISP_PULSE_SEP                    17
#define RSSMA_SENSE_POWER_DISP_TRAN_NEG_DUR                 18
#define RSSMA_SENSE_POWER_DISP_TRAN_NEG_OCC                 19
#define RSSMA_SENSE_POWER_DISP_TRAN_NEG_OVER                20
#define RSSMA_SENSE_POWER_DISP_TRAN_POS_DUR                 21
#define RSSMA_SENSE_POWER_DISP_TRAN_POS_OCC                 22
#define RSSMA_SENSE_POWER_DISP_TRAN_POS_OVER                23
    
/* --- Power Sensor Display Indication ------------------------------------- */     
#define RSSMA_SENSE_POWER_PERM_PRIOR_AVER                   0
#define RSSMA_SENSE_POWER_PERM_PRIOR_PEAK                   1

/* --- Power Sensor Background Color ------------------------------------- */		
#define RSSMA_SENSE_POWER_DISPLAY_BC_BLACK 					0
#define RSSMA_SENSE_POWER_DISPLAY_BC_WHITE 					1
    
/* --- Calibration Level Adjustment ---------------------------------------- */
#define RSSMA_CAL_DATA_FACT                                 0
#define RSSMA_CAL_DATA_CUST                                 1
	
/* --- Time Schema --------------------------------------------------------- */
#define RSSMA_TIME_SCHEMA_STD                               0
#define RSSMA_TIME_SCHEMA_USER                              1
	
/* --- ILS Polarity -------------------------------------------------------- */
#define RSSMA_ILS_DDM_POL_P90_150                           0
#define RSSMA_ILS_DDM_POL_P150_90                           1
	
/* --- List Attenuation Mode ----------------------------------------------- */
#define RSSMA_LIST_ATTEN_MODE_NORM                          0
#define RSSMA_LIST_ATTEN_MODE_HPOW                          1
	
/* --- Sweep Shape --------------------------------------------------------- */
#define RSSMA_POW_SWEEP_SHAPE_SAWT                          0
#define RSSMA_POW_SWEEP_SHAPE_TRI                           1
	
/* --- Sweep Spacing ------------------------------------------------------- */
#define RSSMA_POW_SWEEP_SPACING_LIN                         0
#define RSSMA_POW_SWEEP_SPACING_LOG                         1

/* --- Pulse Impedance ----------------------------------------------------- */
#define RSSMA_EXT_INPUT_IMPEDANCE_1G0                       1
	
/* --- Gate Polarity ------------------------------------------------------- */
#define RSSMA_DME_GATE_POLARITY_NORM                        0
#define RSSMA_DME_GATE_POLARITY_INV                         1

/* --- Power Sensor Sweep Frequency ---------------------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_FREQ_SPACING_LIN 			0
#define RSSMA_SENSE_POWER_SWEEP_FREQ_SPACING_LOG 			1
    
/* --- Power Sensor Sweep Frequency Y Scale -------------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_YSCALE_AUTO_OFF             0
#define RSSMA_SENSE_POWER_SWEEP_YSCALE_AUTO_CEXP            1
#define RSSMA_SENSE_POWER_SWEEP_YSCALE_AUTO_FEXP            2
#define RSSMA_SENSE_POWER_SWEEP_YSCALE_AUTO_CFL             3
#define RSSMA_SENSE_POWER_SWEEP_YSCALE_AUTO_FFL             4

/* --- Power Sensor Sweep Hardcopy Device  --------------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_DEVICE_FILE 		0

/* --- Power Sensor Sweep Hardcopy CSV Decimal Point ----------------------- */
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_DECPOINT_DOT 	0
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_DECPOINT_COM 	1

/* --- Power Sensor Sweep Hardcopy CSV Header ------------------------------ */
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_HEADER_OFF		0
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_HEADER_STD		1
	
/* --- Power Sensor Sweep Hardcopy CSV Orientation ------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_ORIENT_HOR 	0
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_ORIENT_VER 	1

/* --- Power Sensor Sweep Hardcopy CSV Column Separator -------------------- */
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_COL_TAB 		0
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_COL_SEM 		1
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_COL_COM 		2
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_CSV_COL_BLN 		3

/* --- Pulse Data Threshold Base ------------------------------------------- */
#define RSSMA_SENSE_POWER_TIME_THRESHOLD_BASE_VOLT 			0
#define RSSMA_SENSE_POWER_TIME_THRESHOLD_BASE_POW  			1
	
/* --- Trace Color --------------------------------------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_INVE 			0
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_GRAY 			1
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_YELL 			2
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_BLUE 			3
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_GREE 			4
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_RED  			5
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COLOR_MAGN 			6

/* --- Trace Copy ---------------------------------------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_TRACE_COPY_REF				0
	
/* --- Trace Source -------------------------------------------------------- */
#define RSSMA_SENSE_POWER_SWEEP_TRACE_SOURCE_S1  			0
#define RSSMA_SENSE_POWER_SWEEP_TRACE_SOURCE_S2  			1
#define RSSMA_SENSE_POWER_SWEEP_TRACE_SOURCE_S3  			2
#define RSSMA_SENSE_POWER_SWEEP_TRACE_SOURCE_REF 			3
#define RSSMA_SENSE_POWER_SWEEP_TRACE_SOURCE_NON 			4
	
#define RSSMA_SENSE_POWER_SWEEP_TIMING_NORM                 0
#define RSSMA_SENSE_POWER_SWEEP_TIMING_FAST                 1	

#define RSSMA_SENSE_POWER_SWEEP_SPACING_LIN                 0
#define RSSMA_SENSE_POWER_SWEEP_SPACING_LOG                 1
	
#define RSSMA_SENSE_POWER_SWEEP_RMODE_SING                  0
#define RSSMA_SENSE_POWER_SWEEP_RMODE_CONT                  1
	
#define RSSMA_SENSE_POWER_GATE_TRAC_1                       0
#define RSSMA_SENSE_POWER_GATE_TRAC_2                       1
#define RSSMA_SENSE_POWER_GATE_TRAC_3                       2
	
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_LANG_BMP           0
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_LANG_JPG           1
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_LANG_XPM           2
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_LANG_PNG           3
#define RSSMA_SENSE_POWER_SWEEP_HARDCOPY_LANG_CSV           4
	
#define RSSMA_SENSE_POWER_TIME_THRESHOLD_DISTAL             0
#define RSSMA_SENSE_POWER_TIME_THRESHOLD_MESIAL             1
#define RSSMA_SENSE_POWER_TIME_THRESHOLD_PROXIM             2
	
#define RSSMA_SENSE_POWER_SWEEP_FREQ_TIMING_NORM            0
#define RSSMA_SENSE_POWER_SWEEP_FREQ_TIMING_FAST            1
	
#define RSSMA_SENSE_POWER_SWEEP_POWER_SPACING_LIN           0
#define RSSMA_SENSE_POWER_SWEEP_POWER_SPACING_LOG           1
	
#define RSSMA_SENSE_POWER_SWEEP_POWER_TIMING_NORM           0
#define RSSMA_SENSE_POWER_SWEEP_POWER_TIMING_FAST           1
	
#define RSSMA_DM_SOURCE_EXT1 								0
#define RSSMA_DM_SOURCE_EXT2 								1
#define RSSMA_DM_SOURCE_PGEN 								2
#define RSSMA_DM_SOURCE_RAND 								3
	
#define RSSMA_SWEEP_TRIGGER_EVENTS_AND 						0
#define RSSMA_SWEEP_TRIGGER_EVENTS_OR  						1

#define RSSMA_MATH_SUBTRACT_T1T1  							0
#define RSSMA_MATH_SUBTRACT_T1T2  							1
#define RSSMA_MATH_SUBTRACT_T1T3  							2
#define RSSMA_MATH_SUBTRACT_T1T4  							3
#define RSSMA_MATH_SUBTRACT_T1REF 							4
#define RSSMA_MATH_SUBTRACT_T2T1  							5
#define RSSMA_MATH_SUBTRACT_T2T2  							6
#define RSSMA_MATH_SUBTRACT_T2T3  							7
#define RSSMA_MATH_SUBTRACT_T2T4  							8
#define RSSMA_MATH_SUBTRACT_T2REF 							9
#define RSSMA_MATH_SUBTRACT_T3T1  							10
#define RSSMA_MATH_SUBTRACT_T3T2  							11
#define RSSMA_MATH_SUBTRACT_T3T3  							12
#define RSSMA_MATH_SUBTRACT_T3T4  							13
#define RSSMA_MATH_SUBTRACT_T3REF 							14
#define RSSMA_MATH_SUBTRACT_T4T1  							15
#define RSSMA_MATH_SUBTRACT_T4T2  							16
#define RSSMA_MATH_SUBTRACT_T4T3  							17
#define RSSMA_MATH_SUBTRACT_T4T4  							18
#define RSSMA_MATH_SUBTRACT_T4REF 							19
	
#define RSSMA_ROSC_EXTERNAL_FREQ_5MHZ  0
#define RSSMA_ROSC_EXTERNAL_FREQ_10MHZ 1
	
#define RSSMA_POWER_SWEEP_TIME_TRIGGER_EVENT_AND 0
#define RSSMA_POWER_SWEEP_TIME_TRIGGER_EVENT_OR  1
	
/* --- NRP-Z Level Control ------------------------------------------------- */
#define RSSMA_NRPZ_POWER_CONTROL_SENSOR1					0
#define RSSMA_NRPZ_POWER_CONTROL_SENSOR2					1
#define RSSMA_NRPZ_POWER_CONTROL_SENSOR3					2
#define RSSMA_NRPZ_POWER_CONTROL_SENSOR4					3

/* --- Keyboard ------------------------------------------------------------ */
#define RSSMA_KBO_LANG_US									0
#define RSSMA_KBO_LANG_DE									1

/* --- Keyboard Layout------------------------------------------------------ */  
#define RSSMA_KBO_LAY_CHIN								   	0
#define RSSMA_KBO_LAY_DAN   								1
#define RSSMA_KBO_LAY_DUTC 									2
#define RSSMA_KBO_LAY_ENGUK									3
#define RSSMA_KBO_LAY_ENGUS									4
#define RSSMA_KBO_LAY_FINN 									5
#define RSSMA_KBO_LAY_FREN 									6
#define RSSMA_KBO_LAY_FREB 									7
#define RSSMA_KBO_LAY_FREC 									8
#define RSSMA_KBO_LAY_GERM 									9
#define RSSMA_KBO_LAY_ITAL 									10
#define RSSMA_KBO_LAY_JAP  									11
#define RSSMA_KBO_LAY_KOR  									12
#define RSSMA_KBO_LAY_NORW 									13
#define RSSMA_KBO_LAY_PORT 									14
#define RSSMA_KBO_LAY_RUSS 									15
#define RSSMA_KBO_LAY_SPAN 									16
#define RSSMA_KBO_LAY_SWED 									17


/* --- System Language ----------------------------------------------------- */
#define RSSMA_SYSTEM_LANGUAGE_SCPI   						0
#define RSSMA_SYSTEM_LANGUAGE_HP8662 						1
#define RSSMA_SYSTEM_LANGUAGE_HP864X 						2
#define RSSMA_SYSTEM_LANGUAGE_AF2040 						3
#define RSSMA_SYSTEM_LANGUAGE_RC3102 						4


/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
ViStatus _VI_FUNC rssma_init (ViRsrc resourceName, ViBoolean IDQuery,
                              ViBoolean resetDevice, ViPSession instrumentHandle);
ViStatus _VI_FUNC rssma_confLFFreqSweep (ViSession instrumentHandle, ViInt32 mode,
                                         ViReal64 startFrequency,
                                         ViReal64 stopFrequency, ViInt32 spacing,
                                         ViReal64 step, ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_LFSweepTrigger (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetLFSweepMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_SetLFSweepSource (ViSession instrumentHandle,
                                          ViInt32 source);
ViStatus _VI_FUNC rssma_GetLFSweepSource (ViSession instrumentHandle,
                                          ViPInt32 source);
ViStatus _VI_FUNC rssma_SetLFSweepStartFreq (ViSession instrumentHandle,
                                             ViReal64 startFrequency);
ViStatus _VI_FUNC rssma_GetLFSweepStartFreq (ViSession instrumentHandle,
                                             ViPReal64 startFrequency);
ViStatus _VI_FUNC rssma_SetLFSweepStopFreq (ViSession instrumentHandle,
                                            ViReal64 stopFrequency);
ViStatus _VI_FUNC rssma_GetLFSweepStopFreq (ViSession instrumentHandle,
                                            ViPReal64 stopFrequency);
ViStatus _VI_FUNC rssma_SetLFSweepSpacing (ViSession instrumentHandle,
                                           ViInt32 spacing);
ViStatus _VI_FUNC rssma_GetLFSweepSpacing (ViSession instrumentHandle,
                                           ViPInt32 spacing);
ViStatus _VI_FUNC rssma_SetLFSweepStepLin (ViSession instrumentHandle,
                                           ViReal64 stepLin);
ViStatus _VI_FUNC rssma_GetLFSweepStepLin (ViSession instrumentHandle,
                                           ViPReal64 stepLin);
ViStatus _VI_FUNC rssma_SetLFSweepStepLog (ViSession instrumentHandle,
                                           ViReal64 stepLog);
ViStatus _VI_FUNC rssma_GetLFSweepStepLog (ViSession instrumentHandle,
                                           ViPReal64 stepLog);
ViStatus _VI_FUNC rssma_SetLFSweepDwellTime (ViSession instrumentHandle,
                                             ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_GetLFSweepDwellTime (ViSession instrumentHandle,
                                             ViPReal64 dwellTime);
ViStatus _VI_FUNC rssma_SetLFSweepManualStep (ViSession instrumentHandle,
                                              ViReal64 manualLFSweep);
ViStatus _VI_FUNC rssma_GetLFSweepManualStep (ViSession instrumentHandle,
                                              ViPReal64 manualLFSweep);
ViStatus _VI_FUNC rssma_SetLFSweepPoints (ViSession instrumentHandle,
                                          ViInt32 LFSweepPoints);
ViStatus _VI_FUNC rssma_GetLFSweepPoints (ViSession instrumentHandle,
                                          ViPInt32 LFSweepPoints);
ViStatus _VI_FUNC rssma_SetLFSweepShape (ViSession instrumentHandle, ViInt32 shape);
ViStatus _VI_FUNC rssma_GetLFSweepShape (ViSession instrumentHandle,
                                         ViPInt32 shape);
ViStatus _VI_FUNC rssma_confLFFreq (ViSession instrumentHandle, ViBoolean state,
                                    ViInt32 channel, ViReal64 LFGenFrequency,
                                    ViReal64 LFGenVoltage);
ViStatus _VI_FUNC rssma_SetLFOutputState (ViSession instrumentHandle,
                                          ViBoolean LFOutputState);
ViStatus _VI_FUNC rssma_GetLFOutputState (ViSession instrumentHandle,
                                          ViPBoolean LFOutputState);
ViStatus _VI_FUNC rssma_SetLFGenSource (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rssma_GetLFGenSource (ViSession instrumentHandle,
                                        ViPInt32 source);
ViStatus _VI_FUNC rssma_SetLFGenVoltage (ViSession instrumentHandle,
                                         ViReal64 LFGenVoltage);
ViStatus _VI_FUNC rssma_GetLFGenVoltage (ViSession instrumentHandle,
                                         ViPReal64 LFGenVoltage);
ViStatus _VI_FUNC rssma_SetLFGenInternalVoltage (ViSession instrumentHandle,
                                                 ViInt32 channel,
                                                 ViReal64 LFGenInternalVoltage);
ViStatus _VI_FUNC rssma_GetLFGenInternalVoltage (ViSession instrumentHandle,
                                                 ViInt32 channel,
                                                 ViPReal64 LFGenInternalVoltage);
ViStatus _VI_FUNC rssma_SetLFGenFrequency (ViSession instrumentHandle,
                                           ViInt32 channel,
                                           ViReal64 LFGenFrequency);
ViStatus _VI_FUNC rssma_GetLFGenFrequency (ViSession instrumentHandle,
                                           ViInt32 channel,
                                           ViPReal64 LFGenFrequency);
ViStatus _VI_FUNC rssma_SetLFGenShape (ViSession instrumentHandle, ViInt32 shape);
ViStatus _VI_FUNC rssma_GetLFGenShape (ViSession instrumentHandle, ViPInt32 shape);
ViStatus _VI_FUNC rssma_SetLFGenShapeTrapeze (ViSession instrumentHandle,
                                              ViInt32 trapeze, ViReal64 time);
ViStatus _VI_FUNC rssma_GetLFGenShapeTrapeze (ViSession instrumentHandle,
                                              ViInt32 trapeze, ViPReal64 time);
ViStatus _VI_FUNC rssma_SetLFGenDCOffset (ViSession instrumentHandle,
                                          ViReal64 DCOffset);
ViStatus _VI_FUNC rssma_GetLFGenDCOffset (ViSession instrumentHandle,
                                          ViPReal64 DCOffset);
ViStatus _VI_FUNC rssma_SetPGENState (ViSession instrumentHandle,
                                      ViBoolean pulseGeneratorState);
ViStatus _VI_FUNC rssma_GetPGENState (ViSession instrumentHandle,
                                      ViPBoolean pulseGeneratorState);
ViStatus _VI_FUNC rssma_SetAllModulationsState (ViSession instrumentHandle,
                                                ViBoolean modulationStates);
ViStatus _VI_FUNC rssma_configAM (ViSession instrumentHandle, ViBoolean AMState,
                                  ViInt32 AMSource, ViReal64 AMDepth,
                                  ViInt32 AMExternalCoupling,
                                  ViInt32 AMInternalSource,
                                  ViReal64 AMInternalDepth1,
                                  ViReal64 LFGen1Frequency, ViInt32 LFGen2Shape,
                                  ViReal64 LFGen2Frequency);
ViStatus _VI_FUNC rssma_SetAMState (ViSession instrumentHandle, ViBoolean AMState);
ViStatus _VI_FUNC rssma_GetAMState (ViSession instrumentHandle, ViPBoolean AMState);
ViStatus _VI_FUNC rssma_SetAMSource (ViSession instrumentHandle, ViInt32 AMSource);
ViStatus _VI_FUNC rssma_GetAMSource (ViSession instrumentHandle, ViPInt32 AMSource);
ViStatus _VI_FUNC rssma_SetAMDepth (ViSession instrumentHandle, ViReal64 AMDepth);
ViStatus _VI_FUNC rssma_GetAMDepth (ViSession instrumentHandle, ViPReal64 AMDepth);
ViStatus _VI_FUNC rssma_SetAMExternalCoupling (ViSession instrumentHandle,
                                               ViInt32 AMExternalCoupling);
ViStatus _VI_FUNC rssma_GetAMExternalCoupling (ViSession instrumentHandle,
                                               ViPInt32 AMExternalCoupling);
ViStatus _VI_FUNC rssma_GetAMSensitivity (ViSession instrumentHandle,
                                          ViPReal64 AMSensitivity);
ViStatus _VI_FUNC rssma_SetAMInternalSource (ViSession instrumentHandle,
                                             ViInt32 AMInternalSource);
ViStatus _VI_FUNC rssma_GetAMInternalSource (ViSession instrumentHandle,
                                             ViPInt32 AMInternalSource);
ViStatus _VI_FUNC rssma_SetAMInternalDepth (ViSession instrumentHandle,
                                            ViInt32 channel,
                                            ViReal64 AMInternalDepth);
ViStatus _VI_FUNC rssma_GetAMInternalDepth (ViSession instrumentHandle,
                                            ViInt32 channel,
                                            ViPReal64 AMInternalDepth);
ViStatus _VI_FUNC rssma_confFM (ViSession instrumentHandle, ViBoolean state,
                                ViInt32 source, ViInt32 mode, ViReal64 FMDeviation,
                                ViReal64 internalDeviation,
                                ViReal64 externalDeviation,
                                ViInt32 externalCoupling,
                                ViInt32 externalInputImpedance,
                                ViInt32 binaryFormat);
ViStatus _VI_FUNC rssma_confFMInternal (ViSession instrumentHandle,
                                        ViInt32 internalSource,
                                        ViReal64 internalDeviation1,
                                        ViReal64 LFGen1Frequency,
                                        ViInt32 LFGen2Shape,
                                        ViReal64 LFGen2Frequency);
ViStatus _VI_FUNC rssma_SetFMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetFMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetFMSource (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rssma_GetFMSource (ViSession instrumentHandle, ViPInt32 source);
ViStatus _VI_FUNC rssma_SetFMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetFMMode (ViSession instrumentHandle, ViPInt32 mode);
ViStatus _VI_FUNC rssma_SetFMDeviation (ViSession instrumentHandle,
                                        ViReal64 externalDeviation);
ViStatus _VI_FUNC rssma_GetFMDeviation (ViSession instrumentHandle,
                                        ViPReal64 externalDeviation);
ViStatus _VI_FUNC rssma_SetFMExtCoupling (ViSession instrumentHandle,
                                          ViInt32 externalCoupling);
ViStatus _VI_FUNC rssma_GetFMExtCoupling (ViSession instrumentHandle,
                                          ViPInt32 externalCoupling);
ViStatus _VI_FUNC rssma_SetFMInternalSource (ViSession instrumentHandle,
                                             ViInt32 internalSource);
ViStatus _VI_FUNC rssma_GetFMInternalSource (ViSession instrumentHandle,
                                             ViPInt32 internalSource);
ViStatus _VI_FUNC rssma_SetFMInternalDeviation (ViSession instrumentHandle,
                                                ViInt32 channel,
                                                ViReal64 internalDeviation);
ViStatus _VI_FUNC rssma_GetFMInternalDeviation (ViSession instrumentHandle,
                                                ViInt32 channel,
                                                ViPReal64 internalDeviation);
ViStatus _VI_FUNC rssma_SetFMExternalDeviation (ViSession instrumentHandle,
                                                ViReal64 externalDeviation);
ViStatus _VI_FUNC rssma_GetFMExternalDeviation (ViSession instrumentHandle,
                                                ViPReal64 externalDeviation);
ViStatus _VI_FUNC rssma_SetFMExternalDigitalBinaryFormat (ViSession instrumentHandle,
                                                          ViInt32 binaryFormat);
ViStatus _VI_FUNC rssma_GetFMExternalDigitalBinaryFormat (ViSession instrumentHandle,
                                                          ViPInt32 binaryFormat);
ViStatus _VI_FUNC rssma_GetFMSensitivity (ViSession instrumentHandle,
                                          ViPReal64 sensitivity);
ViStatus _VI_FUNC rssma_confPM (ViSession instrumentHandle, ViBoolean state,
                                ViInt32 source, ViInt32 mode, ViReal64 PMDeviation,
                                ViReal64 internalDeviation,
                                ViReal64 externalDeviation,
                                ViInt32 externalCoupling,
                                ViInt32 externalInputImpedance,
                                ViInt32 binaryFormat);
ViStatus _VI_FUNC rssma_confPMInternal (ViSession instrumentHandle,
                                        ViInt32 internalSource,
                                        ViReal64 internalDeviation1,
                                        ViReal64 LFGen1Frequency,
                                        ViInt32 LFGen2Shape,
                                        ViReal64 LFGen2Frequency);
ViStatus _VI_FUNC rssma_SetPMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPMSource (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rssma_GetPMSource (ViSession instrumentHandle, ViPInt32 source);
ViStatus _VI_FUNC rssma_SetPMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetPMMode (ViSession instrumentHandle, ViPInt32 mode);
ViStatus _VI_FUNC rssma_SetPMDeviation (ViSession instrumentHandle,
                                        ViReal64 deviation);
ViStatus _VI_FUNC rssma_GetPMDeviation (ViSession instrumentHandle,
                                        ViPReal64 deviation);
ViStatus _VI_FUNC rssma_SetPMExtCoupling (ViSession instrumentHandle,
                                          ViInt32 externalCoupling);
ViStatus _VI_FUNC rssma_GetPMExtCoupling (ViSession instrumentHandle,
                                          ViPInt32 externalCoupling);
ViStatus _VI_FUNC rssma_SetPMInternalSource (ViSession instrumentHandle,
                                             ViInt32 internalSource);
ViStatus _VI_FUNC rssma_GetPMInternalSource (ViSession instrumentHandle,
                                             ViPInt32 internalSource);
ViStatus _VI_FUNC rssma_SetPMInternalDeviation (ViSession instrumentHandle,
                                                ViInt32 channel,
                                                ViReal64 internalDeviation);
ViStatus _VI_FUNC rssma_GetPMInternalDeviation (ViSession instrumentHandle,
                                                ViInt32 channel,
                                                ViPReal64 internalDeviation);
ViStatus _VI_FUNC rssma_SetPMExternalDeviation (ViSession instrumentHandle,
                                                ViReal64 externalDeviation);
ViStatus _VI_FUNC rssma_GetPMExternalDeviation (ViSession instrumentHandle,
                                                ViPReal64 externalDeviation);
ViStatus _VI_FUNC rssma_SetPMExternalDigitalBinaryFormat (ViSession instrumentHandle,
                                                          ViInt32 binaryFormat);
ViStatus _VI_FUNC rssma_GetPMExternalDigitalBinaryFormat (ViSession instrumentHandle,
                                                          ViPInt32 binaryFormat);
ViStatus _VI_FUNC rssma_GetPMSensitivity (ViSession instrumentHandle,
                                          ViPReal64 sensitivity);
ViStatus _VI_FUNC rssma_confPulm (ViSession instrumentHandle, ViBoolean state,
                                  ViInt32 source, ViInt32 polarity,
                                  ViBoolean pulseGeneratorState,
                                  ViReal64 pulsePeriod, ViReal64 pulseWidth,
                                  ViReal64 pulseDelay, ViBoolean doublePulseState,
                                  ViReal64 doublePulseWidth,
                                  ViReal64 doublePulseDelay, ViInt32 triggerMode);
ViStatus _VI_FUNC rssma_SetPULMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPULMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPULMPolarity (ViSession instrumentHandle,
                                         ViInt32 polarity);
ViStatus _VI_FUNC rssma_GetPULMPolarity (ViSession instrumentHandle,
                                         ViPInt32 polarity);
ViStatus _VI_FUNC rssma_SetPULMSource (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rssma_GetPULMSource (ViSession instrumentHandle, ViPInt32 source);
ViStatus _VI_FUNC rssma_SetPULMPeriod (ViSession instrumentHandle,
                                       ViReal64 pulsePeriod);
ViStatus _VI_FUNC rssma_GetPULMPeriod (ViSession instrumentHandle,
                                       ViPReal64 pulsePeriod);
ViStatus _VI_FUNC rssma_SetPULMWidth (ViSession instrumentHandle,
                                      ViReal64 pulseWidth);
ViStatus _VI_FUNC rssma_GetPULMWidth (ViSession instrumentHandle,
                                      ViPReal64 pulseWidth);
ViStatus _VI_FUNC rssma_SetPULMDelay (ViSession instrumentHandle,
                                      ViReal64 pulseDelay);
ViStatus _VI_FUNC rssma_GetPULMDelay (ViSession instrumentHandle,
                                      ViPReal64 pulseDelay);
ViStatus _VI_FUNC rssma_SetPULMDoubleState (ViSession instrumentHandle,
                                            ViBoolean doublePulseState);
ViStatus _VI_FUNC rssma_GetPULMDoubleState (ViSession instrumentHandle,
                                            ViPBoolean doublePulseState);
ViStatus _VI_FUNC rssma_SetPULMDoubleWidth (ViSession instrumentHandle,
                                            ViReal64 doublePulseWidth);
ViStatus _VI_FUNC rssma_GetPULMDoubleWidth (ViSession instrumentHandle,
                                            ViPReal64 doublePulseWidth);
ViStatus _VI_FUNC rssma_SetPULMDoubleDelay (ViSession instrumentHandle,
                                            ViReal64 doublePulseDelay);
ViStatus _VI_FUNC rssma_GetPULMDoubleDelay (ViSession instrumentHandle,
                                            ViPReal64 doublePulseDelay);
ViStatus _VI_FUNC rssma_SetPULMTriggerMode (ViSession instrumentHandle,
                                            ViInt32 triggerMode);
ViStatus _VI_FUNC rssma_GetPULMTriggerMode (ViSession instrumentHandle,
                                            ViPInt32 triggerMode);
ViStatus _VI_FUNC rssma_SetPULMExternalTriggerInputImpedance
             (ViSession instrumentHandle, ViInt32 externalInputImpedance);
ViStatus _VI_FUNC rssma_GetPULMExternalTriggerInputImpedance
             (ViSession instrumentHandle, ViPInt32 externalInputImpedance);
ViStatus _VI_FUNC rssma_SetPULMExternalTriggerSlope (ViSession instrumentHandle,
                                                     ViInt32 externalTriggerSlope);
ViStatus _VI_FUNC rssma_GetPULMExternalTriggerSlope (ViSession instrumentHandle,
                                                     ViPInt32 externalTriggerSlope);
ViStatus _VI_FUNC rssma_SetPULMGateInputPolarity (ViSession instrumentHandle,
                                                  ViInt32 gateInputPolarity);
ViStatus _VI_FUNC rssma_GetPULMGateInputPolarity (ViSession instrumentHandle,
                                                  ViPInt32 gateInputPolarity);
ViStatus _VI_FUNC rssma_SetPULMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetPULMMode (ViSession instrumentHandle, ViPInt32 mode);

ViStatus _VI_FUNC rssma_PULMPulseTrainCatalog (ViSession instrumentHandle,
                                               ViString directoryPath,
                                               ViInt32 arraySize,
                                               ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_PULMDeletePulseTrainFile (ViSession instrumentHandle,
                                                  ViString fileName);
ViStatus _VI_FUNC rssma_PULMSelectPulseTrainFile (ViSession instrumentHandle,
                                                  ViString fileName);
ViStatus _VI_FUNC rssma_SetPULMTrainOfftime (ViSession instrumentHandle,
                                             ViInt32 arraySize,
                                             ViReal64 _VI_FAR offtime[]);
ViStatus _VI_FUNC rssma_GetPULMTrainOfftime (ViSession instrumentHandle,
                                             ViInt32 arraySize,
                                             ViReal64 _VI_FAR offtime[]);
ViStatus _VI_FUNC rssma_GetTrainOfftimePoints (ViSession instrumentHandle,
                                               ViPInt32 numberOfOfftimePoints);
ViStatus _VI_FUNC rssma_SetPULMTrainOntime (ViSession instrumentHandle,
                                            ViInt32 arraySize,
                                            ViReal64 _VI_FAR ontime[]);
ViStatus _VI_FUNC rssma_GetPULMTrainOntime (ViSession instrumentHandle,
                                            ViInt32 arraySize,
                                            ViReal64 _VI_FAR ontime[]);
ViStatus _VI_FUNC rssma_GetTrainOntimePoints (ViSession instrumentHandle,
                                              ViPInt32 numberOfOntimePoints);
ViStatus _VI_FUNC rssma_SetPULMTrainRepetition (ViSession instrumentHandle,
                                                ViInt32 arraySize,
                                                ViInt32 _VI_FAR repetition[]);
ViStatus _VI_FUNC rssma_GetPULMTrainRepetition (ViSession instrumentHandle,
                                                ViInt32 arraySize,
                                                ViInt32 _VI_FAR repetition[]);
ViStatus _VI_FUNC rssma_GetPULMTrainRepetitionPoints (ViSession instrumentHandle,
                                                      ViPInt32 numberOfRepetitionPoints);
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportListMode (ViSession instrumentHandle,
                                                          ViInt32 listMode);
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportListMode (ViSession instrumentHandle,
                                                          ViPInt32 listMode);
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportListFile (ViSession instrumentHandle,
                                                          ViString sourceFile);
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportListFile (ViSession instrumentHandle,
                                                          ViChar _VI_FAR sourceFile[]);
ViStatus _VI_FUNC rssma_PULMTrainImportExportExecute (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIFile
             (ViSession instrumentHandle, ViString ASCIISourceFile);
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIFile
             (ViSession instrumentHandle, ViChar _VI_FAR ASCIISourceFile[]);
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIExtension
             (ViSession instrumentHandle, ViInt32 extension);
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIExtension
             (ViSession instrumentHandle, ViPInt32 extension);
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIColumnSeparator
             (ViSession instrumentHandle, ViInt32 separator);
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIColumnSeparator
             (ViSession instrumentHandle, ViPInt32 separator);
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIDecimalSeparator
             (ViSession instrumentHandle, ViInt32 decimalSeparator);
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIDecimalSeparator
             (ViSession instrumentHandle, ViPInt32 decimalSeparator);
ViStatus _VI_FUNC rssma_PULMTrainImportExportCatalog (ViSession instrumentHandle,
                                                      ViInt32 arraySize,
                                                      ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_confDMEM (ViSession instrumentHandle, ViBoolean state,
                                  ViInt32 mode, ViBoolean lowEmission,
                                  ViBoolean singlePulse, ViBoolean pulseSquitter,
                                  ViInt32 rangeDistance, ViInt32 rate);
ViStatus _VI_FUNC rssma_confDMEMEcho (ViSession instrumentHandle,
                                      ViBoolean echoPulse, ViReal64 echoAttenuation,
                                      ViReal64 echoDelay);
ViStatus _VI_FUNC rssma_confDMEMPulse (ViSession instrumentHandle,
                                       ViInt32 channelSuffix, ViInt32 pulseShape,
                                       ViReal64 pulseRise, ViReal64 pulseFall,
                                       ViReal64 pulseWidth, ViReal64 pulseSpacing);
ViStatus _VI_FUNC rssma_confDMEMTrigger (ViSession instrumentHandle,
                                         ViInt32 triggerMode, ViReal64 triggerDelay,
                                         ViInt32 externalInputImpedance,
                                         ViInt32 extTriggerInputSlope);
ViStatus _VI_FUNC rssma_DMEMPreset (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetDMEMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetDMEMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetDMEMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetDMEMMode (ViSession instrumentHandle, ViPInt32 mode);
ViStatus _VI_FUNC rssma_SetDMEMPulseShape (ViSession instrumentHandle,
                                           ViInt32 pulseShape);
ViStatus _VI_FUNC rssma_GetDMEMPulseShape (ViSession instrumentHandle,
                                           ViPInt32 pulseShape);
ViStatus _VI_FUNC rssma_SetDMEMPulseRise (ViSession instrumentHandle,
                                          ViReal64 pulseRise);
ViStatus _VI_FUNC rssma_GetDMEMPulseRise (ViSession instrumentHandle,
                                          ViPReal64 pulseRise);
ViStatus _VI_FUNC rssma_SetDMEMPulseWidth (ViSession instrumentHandle,
                                           ViReal64 pulseWidth);
ViStatus _VI_FUNC rssma_GetDMEMPulseWidth (ViSession instrumentHandle,
                                           ViPReal64 pulseWidth);
ViStatus _VI_FUNC rssma_SetDMEMPulseFall (ViSession instrumentHandle,
                                          ViReal64 pulseFall);
ViStatus _VI_FUNC rssma_GetDMEMPulseFall (ViSession instrumentHandle,
                                          ViPReal64 pulseFall);
ViStatus _VI_FUNC rssma_SetDMEMPulseSpacing (ViSession instrumentHandle,
                                             ViReal64 pulseSpacing);
ViStatus _VI_FUNC rssma_GetDMEMPulseSpacing (ViSession instrumentHandle,
                                             ViPReal64 pulseSpacing);
ViStatus _VI_FUNC rssma_SetDMEMSquitter (ViSession instrumentHandle,
                                         ViBoolean pulseSquitter);
ViStatus _VI_FUNC rssma_GetDMEMSquitter (ViSession instrumentHandle,
                                         ViPBoolean pulseSquitter);
ViStatus _VI_FUNC rssma_SetDMEMRate (ViSession instrumentHandle, ViInt32 rate);
ViStatus _VI_FUNC rssma_GetDMEMRate (ViSession instrumentHandle, ViPInt32 rate);
ViStatus _VI_FUNC rssma_SetDMEMEchoPulse (ViSession instrumentHandle,
                                          ViBoolean echoPulse);
ViStatus _VI_FUNC rssma_GetDMEMEchoPulse (ViSession instrumentHandle,
                                          ViPBoolean echoPulse);
ViStatus _VI_FUNC rssma_SetDMEMEchoAttenuation (ViSession instrumentHandle,
                                                ViReal64 echoAttenuation);
ViStatus _VI_FUNC rssma_GetDMEMEchoAttenuation (ViSession instrumentHandle,
                                                ViPReal64 echoAttenuation);
ViStatus _VI_FUNC rssma_SetDMEMEchoDelay (ViSession instrumentHandle,
                                          ViReal64 echoDelay);
ViStatus _VI_FUNC rssma_GetDMEMEchoDelay (ViSession instrumentHandle,
                                          ViPReal64 echoDelay);
ViStatus _VI_FUNC rssma_SetDMEMTriggerMode (ViSession instrumentHandle,
                                            ViInt32 triggerMode);
ViStatus _VI_FUNC rssma_GetDMEMTriggerMode (ViSession instrumentHandle,
                                            ViPInt32 triggerMode);
ViStatus _VI_FUNC rssma_SetDMEMTriggerDelay (ViSession instrumentHandle,
                                             ViReal64 triggerDelay);
ViStatus _VI_FUNC rssma_GetDMEMTriggerDelay (ViSession instrumentHandle,
                                             ViPReal64 triggerDelay);
ViStatus _VI_FUNC rssma_GetDMEMTriggerNormalizeDelay (ViSession instrumentHandle,
                                                      ViPReal64 triggerNormalizeDelay);
ViStatus _VI_FUNC rssma_SetDMEMGatePolarity (ViSession instrumentHandle,
                                             ViInt32 gatePolarity);
ViStatus _VI_FUNC rssma_GetDMEMGatePolarity (ViSession instrumentHandle,
                                             ViPInt32 gatePolarity);
ViStatus _VI_FUNC rssma_SetDMEMExternalInputImpedance (ViSession instrumentHandle,
                                                       ViInt32 externalInputImpedance);
ViStatus _VI_FUNC rssma_GetDMEMExternalInputImpedance (ViSession instrumentHandle,
                                                       ViPInt32 externalInputImpedance);
ViStatus _VI_FUNC rssma_SetDMEMExtTriggerInputSlope (ViSession instrumentHandle,
                                                     ViInt32 extTriggerInputSlope);
ViStatus _VI_FUNC rssma_GetDMEMExtTriggerInputSlope (ViSession instrumentHandle,
                                                     ViPInt32 extTriggerInputSlope);
ViStatus _VI_FUNC rssma_SetDMEMRangeDistance (ViSession instrumentHandle,
                                              ViInt32 rangeDistance);
ViStatus _VI_FUNC rssma_GetDMEMRangeDistance (ViSession instrumentHandle,
                                              ViPInt32 rangeDistance);
ViStatus _VI_FUNC rssma_SetDMEMLowSpectralEmission (ViSession instrumentHandle,
                                                    ViBoolean lowEmission);
ViStatus _VI_FUNC rssma_GetDMEMLowSpectralEmission (ViSession instrumentHandle,
                                                    ViPBoolean lowEmission);
ViStatus _VI_FUNC rssma_SetDMEMChannelSuffix (ViSession instrumentHandle,
                                              ViInt32 channelSuffix);
ViStatus _VI_FUNC rssma_GetDMEMChannelSuffix (ViSession instrumentHandle,
                                              ViPInt32 channelSuffix);
ViStatus _VI_FUNC rssma_SetDMEMReplyEfficiency (ViSession instrumentHandle,
                                                ViInt32 replyEfficiency);
ViStatus _VI_FUNC rssma_GetDMEMReplyEfficiency (ViSession instrumentHandle,
                                                ViPInt32 replyEfficiency);
ViStatus _VI_FUNC rssma_SetDMEMSinglePulse (ViSession instrumentHandle,
                                            ViBoolean singlePulse);
ViStatus _VI_FUNC rssma_GetDMEMSinglePulse (ViSession instrumentHandle,
                                            ViPBoolean singlePulse);

ViStatus _VI_FUNC rssma_SetDMEMICAOChannel (ViSession instrumentHandle,
                                            ViInt32 ICAOChannel,
                                            ViInt32 ICAONumber);

ViStatus _VI_FUNC rssma_GetDMEMICAOChannel (ViSession instrumentHandle,
                                            ViInt32 *ICAOChannel,
                                            ViInt32 *ICAONumber);

ViStatus _VI_FUNC rssma_SetDMEMCompositeAMState (ViSession instrumentHandle,
                                                 ViBoolean state);

ViStatus _VI_FUNC rssma_GetDMEMCompositeAMState (ViSession instrumentHandle,
                                                 ViBoolean *state);

ViStatus _VI_FUNC rssma_SetDMEMCompositeAMModulationDepth (ViSession instrumentHandle,
                                                           ViInt32 signalComponent,
                                                           ViReal64 modulationDepth);

ViStatus _VI_FUNC rssma_GetDMEMCompositeAMModulationDepth (ViSession instrumentHandle,
                                                           ViInt32 signalComponent,
                                                           ViReal64 *modulationDepth);

ViStatus _VI_FUNC rssma_SetDMEMCompositeAMFrequency (ViSession instrumentHandle,
                                                     ViInt32 signalComponent,
                                                     ViReal64 frequency);

ViStatus _VI_FUNC rssma_GetDMEMCompositeAMFrequency (ViSession instrumentHandle,
                                                     ViInt32 signalComponent,
                                                     ViReal64 *frequency);

ViStatus _VI_FUNC rssma_SetDMEMCompositeAMPhase (ViSession instrumentHandle,
                                                 ViReal64 phase);

ViStatus _VI_FUNC rssma_GetDMEMCompositeAMPhase (ViSession instrumentHandle,
                                                 ViReal64 *phase);

ViStatus _VI_FUNC rssma_GetDMEMAnalysisReplyEfficiency (ViSession instrumentHandle,
                                                        ViPReal64 efficiency);
ViStatus _VI_FUNC rssma_SetDMEMReplyEfficiencyState (ViSession instrumentHandle,
                                                     ViBoolean efficiencyState);
ViStatus _VI_FUNC rssma_GetDMEMReplyEfficiencyState (ViSession instrumentHandle,
                                                     ViPBoolean efficiencyState);
ViStatus _VI_FUNC rssma_GetDMEMReplyEfficiencyValidity (ViSession instrumentHandle,
                                                        ViPInt32 efficiencyValidity);
ViStatus _VI_FUNC rssma_SetDMEMMeasurementCount (ViSession instrumentHandle,
                                                 ViInt32 measurementCount);
ViStatus _VI_FUNC rssma_GetDMEMMeasurementCount (ViSession instrumentHandle,
                                                 ViPInt32 measurementCount);
ViStatus _VI_FUNC rssma_SetDMEMExpectedReplyDelay (ViSession instrumentHandle,
                                                   ViReal64 replyDelay);
ViStatus _VI_FUNC rssma_GetDMEMExpectedReplyDelay (ViSession instrumentHandle,
                                                   ViPReal64 replyDelay);
ViStatus _VI_FUNC rssma_SetDMEMGateLength (ViSession instrumentHandle,
                                           ViReal64 gateLength);
ViStatus _VI_FUNC rssma_GetDMEMGateLength (ViSession instrumentHandle,
                                           ViPReal64 gateLength);
ViStatus _VI_FUNC rssma_GetDMEMNormalize (ViSession instrumentHandle,
                                          ViPReal64 delay);
ViStatus _VI_FUNC rssma_GetDMEMPeakLevel (ViSession instrumentHandle,
                                          ViPReal64 peakLevel);
ViStatus _VI_FUNC rssma_SetDMEMPeakLevelState (ViSession instrumentHandle,
                                               ViBoolean peakLevel);
ViStatus _VI_FUNC rssma_GetDMEMPeakLevelState (ViSession instrumentHandle,
                                               ViPBoolean peakLevel);
ViStatus _VI_FUNC rssma_GetDMEMPeakLevelValidity (ViSession instrumentHandle,
                                                  ViPInt32 peakLevelValidity);
ViStatus _VI_FUNC rssma_GetDMEMPulseRepetitionFrequency (ViSession instrumentHandle,
                                                         ViPReal64 pulseRepetitionFrequency);
ViStatus _VI_FUNC rssma_SetDMEMPulseRepetitionFrequencyState
             (ViSession instrumentHandle, ViBoolean pulseRepFreqStat);
ViStatus _VI_FUNC rssma_GetDMEMPulseRepetitionFrequencyState
             (ViSession instrumentHandle, ViPBoolean pulseRepFreqStat);
ViStatus _VI_FUNC rssma_GetDMEMPulseRepetitionFrequencyValidity
             (ViSession instrumentHandle, ViPInt32 pulseRepetitionFreqValidity);
ViStatus _VI_FUNC rssma_SetDMEMAnalysisState (ViSession instrumentHandle,
                                              ViBoolean analysisState);
ViStatus _VI_FUNC rssma_GetDMEMAnalysisState (ViSession instrumentHandle,
                                              ViPBoolean analysisState);
ViStatus _VI_FUNC rssma_GetDMEMReplyDelay (ViSession instrumentHandle,
                                           ViPReal64 replyDelay);
ViStatus _VI_FUNC rssma_SetDMEMReplyDelayState (ViSession instrumentHandle,
                                                ViBoolean replyDelayState);
ViStatus _VI_FUNC rssma_GetDMEMReplyDelayState (ViSession instrumentHandle,
                                                ViPBoolean replyDelayState);
ViStatus _VI_FUNC rssma_GetDMEMReplyDelayValidity (ViSession instrumentHandle,
                                                   ViPInt32 replyDelayValidity);

ViStatus _VI_FUNC rssma_GetDMEMAnalysisTriggerLevel (ViSession instrumentHandle,
                                                     ViReal64 *triggerLevel);

ViStatus _VI_FUNC rssma_confDMEMId (ViSession instrumentHandle, ViBoolean idState,
                                    ViReal64 idFrequency, ViString idCode,
                                    ViReal64 idDotLength, ViInt32 idPeriod);
ViStatus _VI_FUNC rssma_DMEMIdPreset (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetDMEMIdState (ViSession instrumentHandle,
                                        ViBoolean idState);
ViStatus _VI_FUNC rssma_GetDMEMIdState (ViSession instrumentHandle,
                                        ViPBoolean idState);
ViStatus _VI_FUNC rssma_SetDMEMIdCode (ViSession instrumentHandle, ViString idCode);
ViStatus _VI_FUNC rssma_GetDMEMIdCode (ViSession instrumentHandle,
                                       ViChar _VI_FAR idCode[]);
ViStatus _VI_FUNC rssma_SetDMEMIdTimeSchema (ViSession instrumentHandle,
                                             ViInt32 idTimeSchema);
ViStatus _VI_FUNC rssma_GetDMEMIdTimeSchema (ViSession instrumentHandle,
                                             ViPInt32 idTimeSchema);
ViStatus _VI_FUNC rssma_SetDMEMIdDashLength (ViSession instrumentHandle,
                                             ViReal64 idDashLength);
ViStatus _VI_FUNC rssma_GetDMEMIdDashLength (ViSession instrumentHandle,
                                             ViPReal64 idDashLength);
ViStatus _VI_FUNC rssma_SetDMEMIdDotLength (ViSession instrumentHandle,
                                            ViReal64 idDotLength);
ViStatus _VI_FUNC rssma_GetDMEMIdDotLength (ViSession instrumentHandle,
                                            ViPReal64 idDotLength);
ViStatus _VI_FUNC rssma_SetDMEMIdLetterSpace (ViSession instrumentHandle,
                                              ViReal64 idLetterSpace);
ViStatus _VI_FUNC rssma_GetDMEMIdLetterSpace (ViSession instrumentHandle,
                                              ViPReal64 idLetterSpace);
ViStatus _VI_FUNC rssma_SetDMEMIdSymbolSpace (ViSession instrumentHandle,
                                              ViReal64 idSymbolSpace);
ViStatus _VI_FUNC rssma_GetDMEMIdSymbolSpace (ViSession instrumentHandle,
                                              ViPReal64 idSymbolSpace);
ViStatus _VI_FUNC rssma_SetDMEMIdPeriod (ViSession instrumentHandle,
                                         ViInt32 idPeriod);
ViStatus _VI_FUNC rssma_GetDMEMIdPeriod (ViSession instrumentHandle,
                                         ViPInt32 idPeriod);
ViStatus _VI_FUNC rssma_SetDMEMIdFrequency (ViSession instrumentHandle,
                                            ViReal64 idFrequency);
ViStatus _VI_FUNC rssma_GetDMEMIdFrequency (ViSession instrumentHandle,
                                            ViPReal64 idFrequency);
ViStatus _VI_FUNC rssma_ConfigureVORModulation (ViSession instrumentHandle,
                                                ViBoolean state, ViInt32 mode,
                                                ViReal64 bearingAngle,
                                                ViInt32 direction,
                                                ViReal64 VARREFFrequency,
                                                ViReal64 VARDepth,
                                                ViReal64 subcarrierFrequency,
                                                ViReal64 subcarrierDepth,
                                                ViInt32 REFDeviation);
ViStatus _VI_FUNC rssma_ConfigureVORMCOMID (ViSession instrumentHandle,
                                            ViBoolean COMIDState,
                                            ViString COMIDCode,
                                            ViReal64 COMIDFrequency,
                                            ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetVORMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetVORMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetVORMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetVORMMode (ViSession instrumentHandle, ViPInt32 mode);
ViStatus _VI_FUNC rssma_SetVORMBearingAngle (ViSession instrumentHandle,
                                             ViReal64 bearingAngle);
ViStatus _VI_FUNC rssma_GetVORMBearingAngle (ViSession instrumentHandle,
                                             ViPReal64 bearingAngle);
ViStatus _VI_FUNC rssma_SetVORMDirection (ViSession instrumentHandle,
                                          ViInt32 direction);
ViStatus _VI_FUNC rssma_GetVORMDirection (ViSession instrumentHandle,
                                          ViPInt32 direction);
ViStatus _VI_FUNC rssma_SetVORMVARREFFrequency (ViSession instrumentHandle,
                                                ViReal64 VARREFFrequency);
ViStatus _VI_FUNC rssma_GetVORMVARREFFrequency (ViSession instrumentHandle,
                                                ViPReal64 VARREFFrequency);
ViStatus _VI_FUNC rssma_SetVORMVARDepth (ViSession instrumentHandle,
                                         ViReal64 VARDepth);
ViStatus _VI_FUNC rssma_GetVORMVARDepth (ViSession instrumentHandle,
                                         ViPReal64 VARDepth);
ViStatus _VI_FUNC rssma_SetVORMSubcarrierFrequency (ViSession instrumentHandle,
                                                    ViReal64 subcarrierFrequency);
ViStatus _VI_FUNC rssma_GetVORMSubcarrierFrequency (ViSession instrumentHandle,
                                                    ViPReal64 subcarrierFrequency);
ViStatus _VI_FUNC rssma_SetVORMSubcarrierDepth (ViSession instrumentHandle,
                                                ViReal64 subcarrierDepth);
ViStatus _VI_FUNC rssma_GetVORMSubcarrierDepth (ViSession instrumentHandle,
                                                ViPReal64 subcarrierDepth);
ViStatus _VI_FUNC rssma_SetVORMREFDeviation (ViSession instrumentHandle,
                                             ViInt32 REFDeviation);
ViStatus _VI_FUNC rssma_GetVORMREFDeviation (ViSession instrumentHandle,
                                             ViPInt32 REFDeviation);
ViStatus _VI_FUNC rssma_SetVORMToDefault (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetVORMCOMIDState (ViSession instrumentHandle,
                                           ViBoolean COMIDState);
ViStatus _VI_FUNC rssma_GetVORMCOMIDState (ViSession instrumentHandle,
                                           ViPBoolean COMIDState);
ViStatus _VI_FUNC rssma_SetVORMCOMIDCode (ViSession instrumentHandle,
                                          ViString COMIDCode);
ViStatus _VI_FUNC rssma_GetVORMCOMIDCode (ViSession instrumentHandle,
                                          ViChar _VI_FAR COMIDCode[]);
ViStatus _VI_FUNC rssma_SetVORMCOMIDFrequency (ViSession instrumentHandle,
                                               ViReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_GetVORMCOMIDFrequency (ViSession instrumentHandle,
                                               ViPReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_SetVORMCOMIDDepth (ViSession instrumentHandle,
                                           ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_GetVORMCOMIDDepth (ViSession instrumentHandle,
                                           ViPReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetVORMCOMIDTimeSchema (ViSession instrumentHandle,
                                                ViInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_GetVORMCOMIDTimeSchema (ViSession instrumentHandle,
                                                ViPInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_SetVORMCOMIDDashLength (ViSession instrumentHandle,
                                                ViReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_GetVORMCOMIDDashLength (ViSession instrumentHandle,
                                                ViPReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_SetVORMCOMIDDotLength (ViSession instrumentHandle,
                                               ViReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_GetVORMCOMIDDotLength (ViSession instrumentHandle,
                                               ViPReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_SetVORMCOMIDLetterSpace (ViSession instrumentHandle,
                                                 ViReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_GetVORMCOMIDLetterSpace (ViSession instrumentHandle,
                                                 ViPReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_SetVORMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                 ViReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_GetVORMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                 ViPReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_SetVORMCOMIDPeriod (ViSession instrumentHandle,
                                            ViReal64 idPeriod);
ViStatus _VI_FUNC rssma_GetVORMCOMIDPeriod (ViSession instrumentHandle,
                                            ViPReal64 idPeriod);
ViStatus _VI_FUNC rssma_SetVORMEXTAM (ViSession instrumentHandle, ViInt32 EXTAM);
ViStatus _VI_FUNC rssma_GetVORMEXTAM (ViSession instrumentHandle, ViPInt32 EXTAM);

ViStatus _VI_FUNC rssma_SetVORMCarrierFrequencyStep (ViSession instrumentHandle,
                                                     ViInt32 carrierFrequencyStep);

ViStatus _VI_FUNC rssma_GetVORMCarrierFrequencyStep (ViSession instrumentHandle,
                                                     ViInt32 *carrierFrequencyStep);

ViStatus _VI_FUNC rssma_SetVORMICAOChannel (ViSession instrumentHandle,
                                            ViInt32 ICAOChannel,
                                            ViInt32 ICAONumber);

ViStatus _VI_FUNC rssma_GetVORMICAOChannel (ViSession instrumentHandle,
                                            ViInt32 *ICAOChannel,
                                            ViInt32 *ICAONumber);

ViStatus _VI_FUNC rssma_ConfigureILSGSModulation (ViSession instrumentHandle,
                                                  ViBoolean state, ViInt32 mode,
                                                  ViReal64 DDMDepth,
                                                  ViReal64 DDMCurrent,
                                                  ViReal64 DDMLogarithmic,
                                                  ViInt32 DDMDirection,
                                                  ViReal64 sumOfDepth,
                                                  ViReal64 upFrequency,
                                                  ViReal64 downFrequency,
                                                  ViReal64 upDownPhase);
ViStatus _VI_FUNC rssma_SetILSGSMState (ViSession instrumentHandle,
                                        ViBoolean state);
ViStatus _VI_FUNC rssma_GetILSGSMState (ViSession instrumentHandle,
                                        ViPBoolean state);
ViStatus _VI_FUNC rssma_SetILSGSMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetILSGSMMode (ViSession instrumentHandle, ViPInt32 mode);
ViStatus _VI_FUNC rssma_SetILSGSMDDMDepth (ViSession instrumentHandle,
                                           ViReal64 DDMDepth);
ViStatus _VI_FUNC rssma_GetILSGSMDDMDepth (ViSession instrumentHandle,
                                           ViPReal64 DDMDepth);
ViStatus _VI_FUNC rssma_SetILSGSMDDMCurrent (ViSession instrumentHandle,
                                             ViReal64 DDMCurrent);
ViStatus _VI_FUNC rssma_GetILSGSMDDMCurrent (ViSession instrumentHandle,
                                             ViPReal64 DDMCurrent);
ViStatus _VI_FUNC rssma_SetILSGSMDDMLogarithmic (ViSession instrumentHandle,
                                                 ViReal64 DDMLogarithmic);
ViStatus _VI_FUNC rssma_GetILSGSMDDMLogarithmic (ViSession instrumentHandle,
                                                 ViPReal64 DDMLogarithmic);
ViStatus _VI_FUNC rssma_SetILSGSMDDMCoupling (ViSession instrumentHandle,
                                              ViInt32 DDMCoupling);
ViStatus _VI_FUNC rssma_GetILSGSMDDMCoupling (ViSession instrumentHandle,
                                              ViPInt32 DDMCoupling);
ViStatus _VI_FUNC rssma_SetILSGSMDDMPercent (ViSession instrumentHandle,
                                             ViInt32 DDMPercent);
ViStatus _VI_FUNC rssma_GetILSGSMDDMPercent (ViSession instrumentHandle,
                                             ViPInt32 DDMPercent);
ViStatus _VI_FUNC rssma_SetILSGSMDDMPolarity (ViSession instrumentHandle,
                                              ViInt32 DDMPolarity);
ViStatus _VI_FUNC rssma_GetILSGSMDDMPolarity (ViSession instrumentHandle,
                                              ViPInt32 DDMPolarity);
ViStatus _VI_FUNC rssma_SetILSGSMFly (ViSession instrumentHandle,
                                      ViInt32 DDMDirection);
ViStatus _VI_FUNC rssma_GetILSGSMFly (ViSession instrumentHandle,
                                      ViPInt32 DDMDirection);
ViStatus _VI_FUNC rssma_SetILSGSMSumOfDepth (ViSession instrumentHandle,
                                             ViReal64 sumOfDepth);
ViStatus _VI_FUNC rssma_GetILSGSMSumOfDepth (ViSession instrumentHandle,
                                             ViPReal64 sumOfDepth);
ViStatus _VI_FUNC rssma_SetILSGSMUpFrequency (ViSession instrumentHandle,
                                              ViReal64 upFrequency);
ViStatus _VI_FUNC rssma_GetILSGSMUpFrequency (ViSession instrumentHandle,
                                              ViPReal64 upFrequency);
ViStatus _VI_FUNC rssma_SetILSGSMDownFrequency (ViSession instrumentHandle,
                                                ViReal64 downFrequency);
ViStatus _VI_FUNC rssma_GetILSGSMDownFrequency (ViSession instrumentHandle,
                                                ViPReal64 downFrequency);
ViStatus _VI_FUNC rssma_SetILSGSMUpDownPhase (ViSession instrumentHandle,
                                              ViReal64 upDownPhase);
ViStatus _VI_FUNC rssma_GetILSGSMUpDownPhase (ViSession instrumentHandle,
                                              ViPReal64 upDownPhase);
ViStatus _VI_FUNC rssma_SetILSGSMToDefault (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetILSGSMCarrierFrequencyStep (ViSession instrumentHandle,
                                                       ViInt32 carrierFrequencyStep);
ViStatus _VI_FUNC rssma_GetILSGSMCarrierFrequencyStep (ViSession instrumentHandle,
                                                       ViPInt32 carrierFrequencyStep);
ViStatus _VI_FUNC rssma_SetILSGSMICAOChannel (ViSession instrumentHandle,
                                              ViInt32 ICAOChannel);
ViStatus _VI_FUNC rssma_GetILSGSMICAOChannel (ViSession instrumentHandle,
                                              ViPInt32 ICAOChannel);
ViStatus _VI_FUNC rssma_SetILSGSMEXTAM (ViSession instrumentHandle, ViInt32 EXTAM);
ViStatus _VI_FUNC rssma_GetILSGSMEXTAM (ViSession instrumentHandle, ViPInt32 EXTAM);
ViStatus _VI_FUNC rssma_ConfigureILSLOCModulation (ViSession instrumentHandle,
                                                   ViBoolean state, ViInt32 mode,
                                                   ViReal64 DDMDepth,
                                                   ViReal64 DDMCurrent,
                                                   ViReal64 DDMLogarithmic,
                                                   ViInt32 DDMDirection,
                                                   ViReal64 sumOfDepth,
                                                   ViReal64 leftFrequency,
                                                   ViReal64 rightFrequency,
                                                   ViReal64 leftRightPhase);
ViStatus _VI_FUNC rssma_ConfigureILSLOCMCOMID (ViSession instrumentHandle,
                                               ViBoolean COMIDState,
                                               ViString COMIDCode,
                                               ViReal64 COMIDFrequency,
                                               ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetILSLOCMState (ViSession instrumentHandle,
                                         ViBoolean state);
ViStatus _VI_FUNC rssma_GetILSLOCMState (ViSession instrumentHandle,
                                         ViPBoolean state);
ViStatus _VI_FUNC rssma_SetILSLOCMMode (ViSession instrumentHandle, ViInt32 mode);
ViStatus _VI_FUNC rssma_GetILSLOCMMode (ViSession instrumentHandle, ViPInt32 mode);
ViStatus _VI_FUNC rssma_SetILSLOCMDDMDepth (ViSession instrumentHandle,
                                            ViReal64 DDMDepth);
ViStatus _VI_FUNC rssma_GetILSLOCMDDMDepth (ViSession instrumentHandle,
                                            ViPReal64 DDMDepth);
ViStatus _VI_FUNC rssma_SetILSLOCMDDMCurrent (ViSession instrumentHandle,
                                              ViReal64 DDMCurrent);
ViStatus _VI_FUNC rssma_GetILSLOCMDDMCurrent (ViSession instrumentHandle,
                                              ViPReal64 DDMCurrent);
ViStatus _VI_FUNC rssma_SetILSLOCMDDMLogarithmic (ViSession instrumentHandle,
                                                  ViReal64 DDMLogarithmic);
ViStatus _VI_FUNC rssma_GetILSLOCMDDMLogarithmic (ViSession instrumentHandle,
                                                  ViPReal64 DDMLogarithmic);
ViStatus _VI_FUNC rssma_SetILSLOCMDDMCoupling (ViSession instrumentHandle,
                                               ViInt32 DDMCoupling);
ViStatus _VI_FUNC rssma_GetILSLOCMDDMCoupling (ViSession instrumentHandle,
                                               ViPInt32 DDMCoupling);
ViStatus _VI_FUNC rssma_SetILSLOCMDDMPercent (ViSession instrumentHandle,
                                              ViInt32 DDMPercent);
ViStatus _VI_FUNC rssma_GetILSLOCMDDMPercent (ViSession instrumentHandle,
                                              ViPInt32 DDMPercent);
ViStatus _VI_FUNC rssma_SetILSLOCMDDMPolarity (ViSession instrumentHandle,
                                               ViInt32 DDMPolarity);
ViStatus _VI_FUNC rssma_GetILSLOCMDDMPolarity (ViSession instrumentHandle,
                                               ViPInt32 DDMPolarity);
ViStatus _VI_FUNC rssma_SetILSLOCMFly (ViSession instrumentHandle,
                                       ViInt32 DDMDirection);
ViStatus _VI_FUNC rssma_GetILSLOCMFly (ViSession instrumentHandle,
                                       ViPInt32 DDMDirection);
ViStatus _VI_FUNC rssma_SetILSLOCMSumOfDepth (ViSession instrumentHandle,
                                              ViReal64 sumOfDepth);
ViStatus _VI_FUNC rssma_GetILSLOCMSumOfDepth (ViSession instrumentHandle,
                                              ViPReal64 sumOfDepth);
ViStatus _VI_FUNC rssma_SetILSLOCMLeftFrequency (ViSession instrumentHandle,
                                                 ViReal64 leftFrequency);
ViStatus _VI_FUNC rssma_GetILSLOCMLeftFrequency (ViSession instrumentHandle,
                                                 ViPReal64 leftFrequency);
ViStatus _VI_FUNC rssma_SetILSLOCMRightFrequency (ViSession instrumentHandle,
                                                  ViReal64 rightFrequency);
ViStatus _VI_FUNC rssma_GetILSLOCMRightFrequency (ViSession instrumentHandle,
                                                  ViPReal64 rightFrequency);
ViStatus _VI_FUNC rssma_SetILSLOCMLeftRightPhase (ViSession instrumentHandle,
                                                  ViReal64 leftRightPhase);
ViStatus _VI_FUNC rssma_GetILSLOCMLeftRightPhase (ViSession instrumentHandle,
                                                  ViPReal64 leftRightPhase);
ViStatus _VI_FUNC rssma_SetILSLOCMToDefault (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDState (ViSession instrumentHandle,
                                              ViBoolean COMIDState);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDState (ViSession instrumentHandle,
                                              ViPBoolean COMIDState);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDCode (ViSession instrumentHandle,
                                             ViString COMIDCode);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDCode (ViSession instrumentHandle,
                                             ViChar _VI_FAR COMIDCode[]);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDFrequency (ViSession instrumentHandle,
                                                  ViReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDFrequency (ViSession instrumentHandle,
                                                  ViPReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDDepth (ViSession instrumentHandle,
                                              ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDDepth (ViSession instrumentHandle,
                                              ViPReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDTimeSchema (ViSession instrumentHandle,
                                                   ViInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDTimeSchema (ViSession instrumentHandle,
                                                   ViPInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDDashLength (ViSession instrumentHandle,
                                                   ViReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDDashLength (ViSession instrumentHandle,
                                                   ViPReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDDotLength (ViSession instrumentHandle,
                                                  ViReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDDotLength (ViSession instrumentHandle,
                                                  ViPReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDLetterSpace (ViSession instrumentHandle,
                                                    ViReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDLetterSpace (ViSession instrumentHandle,
                                                    ViPReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                    ViReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                    ViPReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_SetILSLOCCOMIDPeriod (ViSession instrumentHandle,
                                              ViInt32 idPeriod);
ViStatus _VI_FUNC rssma_GetILSLOCCOMIDPeriod (ViSession instrumentHandle,
                                              ViPInt32 idPeriod);
ViStatus _VI_FUNC rssma_SetILSLOCMCarrierFrequencyStep (ViSession instrumentHandle,
                                                        ViInt32 carrierFrequencyStep);
ViStatus _VI_FUNC rssma_GetILSLOCMCarrierFrequencyStep (ViSession instrumentHandle,
                                                        ViPInt32 carrierFrequencyStep);
ViStatus _VI_FUNC rssma_SetILSLOCMICAOChannel (ViSession instrumentHandle,
                                               ViInt32 ICAOChannel);
ViStatus _VI_FUNC rssma_GetILSLOCMICAOChannel (ViSession instrumentHandle,
                                               ViPInt32 ICAOChannel);
ViStatus _VI_FUNC rssma_SetILSLOCMEXTAM (ViSession instrumentHandle, ViInt32 EXTAM);
ViStatus _VI_FUNC rssma_GetILSLOCMEXTAM (ViSession instrumentHandle,
                                         ViPInt32 EXTAM);
ViStatus _VI_FUNC rssma_ConfigureMBModulation (ViSession instrumentHandle,
                                               ViBoolean state,
                                               ViInt32 markerFrequency,
                                               ViReal64 markerDepth,
                                               ViBoolean pulsedMarker);
ViStatus _VI_FUNC rssma_ConfigureMBMCOMID (ViSession instrumentHandle,
                                           ViBoolean COMIDState, ViString COMIDCode,
                                           ViReal64 COMIDFrequency,
                                           ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetMBMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetMBMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetMBMMarkerFrequency (ViSession instrumentHandle,
                                               ViInt32 markerFrequency);
ViStatus _VI_FUNC rssma_GetMBMMarkerFrequency (ViSession instrumentHandle,
                                               ViPInt32 markerFrequency);
ViStatus _VI_FUNC rssma_SetMBMMarkerDepth (ViSession instrumentHandle,
                                           ViReal64 markerDepth);
ViStatus _VI_FUNC rssma_GetMBMMarkerDepth (ViSession instrumentHandle,
                                           ViPReal64 markerDepth);
ViStatus _VI_FUNC rssma_SetMBMPulsedMarker (ViSession instrumentHandle,
                                            ViBoolean pulsedMarker);
ViStatus _VI_FUNC rssma_GetMBMPulsedMarker (ViSession instrumentHandle,
                                            ViPBoolean pulsedMarker);
ViStatus _VI_FUNC rssma_SetMBMCOMIDState (ViSession instrumentHandle,
                                          ViBoolean COMIDState);
ViStatus _VI_FUNC rssma_GetMBMCOMIDState (ViSession instrumentHandle,
                                          ViPBoolean COMIDState);
ViStatus _VI_FUNC rssma_SetMBMCOMIDCode (ViSession instrumentHandle,
                                         ViString COMIDCode);
ViStatus _VI_FUNC rssma_GetMBMCOMIDCode (ViSession instrumentHandle,
                                         ViChar _VI_FAR COMIDCode[]);
ViStatus _VI_FUNC rssma_SetMBMCOMIDFrequency (ViSession instrumentHandle,
                                              ViReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_GetMBMCOMIDFrequency (ViSession instrumentHandle,
                                              ViPReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_SetMBMCOMIDDepth (ViSession instrumentHandle,
                                          ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_GetMBMCOMIDDepth (ViSession instrumentHandle,
                                          ViPReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetMBMCOMIDTimeSchema (ViSession instrumentHandle,
                                               ViInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_GetMBMCOMIDTimeSchema (ViSession instrumentHandle,
                                               ViPInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_SetMBMCOMIDDashLength (ViSession instrumentHandle,
                                               ViReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_GetMBMCOMIDDashLength (ViSession instrumentHandle,
                                               ViPReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_SetMBMCOMIDDotLength (ViSession instrumentHandle,
                                              ViReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_GetMBMCOMIDDotLength (ViSession instrumentHandle,
                                              ViPReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_SetMBMCOMIDLetterSpace (ViSession instrumentHandle,
                                                ViReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_GetMBMCOMIDLetterSpace (ViSession instrumentHandle,
                                                ViPReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_SetMBMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                ViReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_GetMBMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                ViPReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_SetMBMCOMIDPeriod (ViSession instrumentHandle,
                                           ViReal64 idPeriod);
ViStatus _VI_FUNC rssma_GetMBMCOMIDPeriod (ViSession instrumentHandle,
                                           ViPReal64 idPeriod);
ViStatus _VI_FUNC rssma_SetMBMToDefault (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_ConfigureADFMCOMID (ViSession instrumentHandle,
                                            ViBoolean COMIDState,
                                            ViString COMIDCode,
                                            ViReal64 COMIDFrequency,
                                            ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetADFMState (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetADFMState (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetADFMCOMIDState (ViSession instrumentHandle,
                                           ViBoolean COMIDState);
ViStatus _VI_FUNC rssma_GetADFMCOMIDState (ViSession instrumentHandle,
                                           ViPBoolean COMIDState);
ViStatus _VI_FUNC rssma_SetADFMCOMIDCode (ViSession instrumentHandle,
                                          ViString COMIDCode);
ViStatus _VI_FUNC rssma_GetADFMCOMIDCode (ViSession instrumentHandle,
                                          ViChar _VI_FAR COMIDCode[]);
ViStatus _VI_FUNC rssma_SetADFMCOMIDFrequency (ViSession instrumentHandle,
                                               ViReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_GetADFMCOMIDFrequency (ViSession instrumentHandle,
                                               ViPReal64 COMIDFrequency);
ViStatus _VI_FUNC rssma_SetADFMCOMIDDepth (ViSession instrumentHandle,
                                           ViReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_GetADFMCOMIDDepth (ViSession instrumentHandle,
                                           ViPReal64 COMIDDepth);
ViStatus _VI_FUNC rssma_SetADFMCOMIDTimeSchema (ViSession instrumentHandle,
                                                ViInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_GetADFMCOMIDTimeSchema (ViSession instrumentHandle,
                                                ViPInt32 COMIDTimeSchema);
ViStatus _VI_FUNC rssma_SetADFMCOMIDDashLength (ViSession instrumentHandle,
                                                ViReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_GetADFMCOMIDDashLength (ViSession instrumentHandle,
                                                ViPReal64 COMIDDashLength);
ViStatus _VI_FUNC rssma_SetADFMCOMIDDotLength (ViSession instrumentHandle,
                                               ViReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_GetADFMCOMIDDotLength (ViSession instrumentHandle,
                                               ViPReal64 COMIDDotLength);
ViStatus _VI_FUNC rssma_SetADFMCOMIDLetterSpace (ViSession instrumentHandle,
                                                 ViReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_GetADFMCOMIDLetterSpace (ViSession instrumentHandle,
                                                 ViPReal64 COMIDLetterSpace);
ViStatus _VI_FUNC rssma_SetADFMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                 ViReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_GetADFMCOMIDSymbolSpace (ViSession instrumentHandle,
                                                 ViPReal64 COMIDSymbolSpace);
ViStatus _VI_FUNC rssma_SetADFMCOMIDPeriod (ViSession instrumentHandle,
                                            ViInt32 idPeriod);
ViStatus _VI_FUNC rssma_GetADFMCOMIDPeriod (ViSession instrumentHandle,
                                            ViPInt32 idPeriod);
ViStatus _VI_FUNC rssma_confCHIRP (ViSession instrumentHandle, ViBoolean CHRIPState,
                                   ViInt32 direction, ViReal64 bandwidth,
                                   ViReal64 pulsePeriod, ViReal64 pulseWidth,
                                   ViInt32 triggerMode, ViInt32 triggerSlope,
                                   ViInt32 impedance);
ViStatus _VI_FUNC rssma_confCHIRPTrigger (ViSession instrumentHandle,
                                          ViInt32 triggerSlope,
                                          ViInt32 gatePolarity, ViInt32 impedance,
                                          ViInt32 triggerMode,
                                          ViInt32 triggerLevel);
ViStatus _VI_FUNC rssma_SetCHIRPTriggerMode (ViSession instrumentHandle,
                                             ViInt32 triggerMode);
ViStatus _VI_FUNC rssma_GetCHIRPTriggerMode (ViSession instrumentHandle,
                                             ViPInt32 triggerMode);
ViStatus _VI_FUNC rssma_SetCHIRPTriggerSlope (ViSession instrumentHandle,
                                              ViInt32 triggerSlope);
ViStatus _VI_FUNC rssma_GetCHIRPTriggerSlope (ViSession instrumentHandle,
                                              ViPInt32 triggerSlope);
ViStatus _VI_FUNC rssma_SetCHIRPGatePolarity (ViSession instrumentHandle,
                                              ViInt32 gatePolarity);
ViStatus _VI_FUNC rssma_GetCHIRPGatePolarity (ViSession instrumentHandle,
                                              ViPInt32 gatePolarity);
ViStatus _VI_FUNC rssma_SetCHIRPExternalImpedance (ViSession instrumentHandle,
                                                   ViInt32 impedance);
ViStatus _VI_FUNC rssma_GetCHIRPExternalImpedance (ViSession instrumentHandle,
                                                   ViPInt32 impedance);
ViStatus _VI_FUNC rssma_SetCHIRPDirection (ViSession instrumentHandle,
                                           ViInt32 direction);
ViStatus _VI_FUNC rssma_GetCHIRPDirection (ViSession instrumentHandle,
                                           ViPInt32 direction);
ViStatus _VI_FUNC rssma_SetCHIRPState (ViSession instrumentHandle,
                                       ViBoolean CHRIPState);
ViStatus _VI_FUNC rssma_GetCHIRPState (ViSession instrumentHandle,
                                       ViPBoolean CHRIPState);
ViStatus _VI_FUNC rssma_SetCHIRPBandwidth (ViSession instrumentHandle,
                                           ViReal64 bandwidth);
ViStatus _VI_FUNC rssma_GetCHIRPBandwidth (ViSession instrumentHandle,
                                           ViPReal64 bandwidth);
ViStatus _VI_FUNC rssma_SetCHIRPPulsePeriod (ViSession instrumentHandle,
                                             ViReal64 pulsePeriod);
ViStatus _VI_FUNC rssma_GetCHIRPPulsePeriod (ViSession instrumentHandle,
                                             ViPReal64 pulsePeriod);
ViStatus _VI_FUNC rssma_SetCHIRPPulseWidth (ViSession instrumentHandle,
                                            ViReal64 pulseWidth);
ViStatus _VI_FUNC rssma_GetCHIRPPulseWidth (ViSession instrumentHandle,
                                            ViPReal64 pulseWidth);
ViStatus _VI_FUNC rssma_GetCHIRPCompressionRatio (ViSession instrumentHandle,
                                                  ViPReal64 compressionRatio);
ViStatus _VI_FUNC rssma_SetAllRFOutputsState (ViSession instrumentHandle,
                                              ViBoolean RFOutputStates);
ViStatus _VI_FUNC rssma_SetOutputState (ViSession instrumentHandle,
                                        ViBoolean RFOutputState);
ViStatus _VI_FUNC rssma_GetOutputState (ViSession instrumentHandle,
                                        ViPBoolean RFOutputState);
ViStatus _VI_FUNC rssma_GetOutputImpedance (ViSession instrumentHandle,
                                            ViPInt32 impedance);
ViStatus _VI_FUNC rssma_confRFFreq (ViSession instrumentHandle, ViReal64 frequency,
                                    ViReal64 offset, ViBoolean variationActive,
                                    ViReal64 variationStep);
ViStatus _VI_FUNC rssma_ConfigureRFPhaseContinuousFrequencySettings
             (ViSession instrumentHandle, ViBoolean state, ViInt32 frequencyRange);
ViStatus _VI_FUNC rssma_SetRFFrequency (ViSession instrumentHandle,
                                        ViReal64 frequency);
ViStatus _VI_FUNC rssma_GetRFFrequency (ViSession instrumentHandle,
                                        ViPReal64 frequency);
ViStatus _VI_FUNC rssma_SetRFFrequencyOffset (ViSession instrumentHandle,
                                              ViReal64 frequencyOffset);
ViStatus _VI_FUNC rssma_GetRFFrequencyOffset (ViSession instrumentHandle,
                                              ViPReal64 frequencyOffset);

ViStatus _VI_FUNC rssma_SetRFMultiplier (ViSession instrumentHandle,
                                         ViReal64 multiplier);

ViStatus _VI_FUNC rssma_GetRFMultiplier (ViSession instrumentHandle,
                                         ViReal64 *multiplier);

ViStatus _VI_FUNC rssma_SetRFPhaseContinuousActive (ViSession instrumentHandle,
                                                    ViBoolean state);
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousActive (ViSession instrumentHandle,
                                                    ViPBoolean state);
ViStatus _VI_FUNC rssma_SetRFPhaseContinuousFrequencyRange
             (ViSession instrumentHandle, ViInt32 frequencyRange);
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousFrequencyRange
             (ViSession instrumentHandle, ViPInt32 frequencyRange);
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousMinimumFrequency
             (ViSession instrumentHandle, ViPReal64 minimumFrequency);
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousMaximumFrequency
             (ViSession instrumentHandle, ViPReal64 maximumFrequency);
ViStatus _VI_FUNC rssma_SetRFVariationActive (ViSession instrumentHandle,
                                              ViBoolean variationActive);
ViStatus _VI_FUNC rssma_GetRFVariationActive (ViSession instrumentHandle,
                                              ViPBoolean variationActive);
ViStatus _VI_FUNC rssma_SetRFVariationStep (ViSession instrumentHandle,
                                            ViReal64 variationStep);
ViStatus _VI_FUNC rssma_GetRFVariationStep (ViSession instrumentHandle,
                                            ViPReal64 variationStep);
ViStatus _VI_FUNC rssma_SetRFFrequencyonRecall (ViSession instrumentHandle,
                                                ViInt32 RFFrequencyOnRecall);
ViStatus _VI_FUNC rssma_GetRFFrequencyonRecall (ViSession instrumentHandle,
                                                ViPInt32 RFFrequencyOnRecall);
ViStatus _VI_FUNC rssma_confRosc (ViSession instrumentHandle, ViInt32 ROSCSource,
                                  ViInt32 externalReferenceFrequency,
                                  ViInt32 ROSCSynchronizationBandwidth,
                                  ViBoolean adjustmentActive,
                                  ViInt32 adjustmentFrequency);
ViStatus _VI_FUNC rssma_SetROSCSource (ViSession instrumentHandle,
                                       ViInt32 ROSCSource);
ViStatus _VI_FUNC rssma_GetROSCSource (ViSession instrumentHandle,
                                       ViPInt32 ROSCSource);
ViStatus _VI_FUNC rssma_SetROSCRFOutput (ViSession instrumentHandle,
                                         ViBoolean RFOutput);
ViStatus _VI_FUNC rssma_GetROSCRFOutput (ViSession instrumentHandle,
                                         ViPBoolean RFOutput);
ViStatus _VI_FUNC rssma_SetROSCExternalReferenceFrequency
             (ViSession instrumentHandle, ViInt32 externalReferenceFrequency);
ViStatus _VI_FUNC rssma_GetROSCExternalReferenceFrequency
             (ViSession instrumentHandle, ViPInt32 externalReferenceFrequency);
ViStatus _VI_FUNC rssma_SetROSCSynchronizationBandwidth (ViSession instrumentHandle,
                                                         ViInt32 ROSCSynchronizationBandwidth);
ViStatus _VI_FUNC rssma_GetROSCSynchronizationBandwidth (ViSession instrumentHandle,
                                                         ViPInt32 ROSCSynchronizationBandwidth);
ViStatus _VI_FUNC rssma_SetROSCExternalFrequency (ViSession instrumentHandle,
                                                  ViInt32 externalFrequency);
ViStatus _VI_FUNC rssma_GetROSCExternalFrequency (ViSession instrumentHandle,
                                                  ViPInt32 externalFrequency);
ViStatus _VI_FUNC rssma_SetROSCAdjustmentActive (ViSession instrumentHandle,
                                                 ViBoolean adjustmentActive);
ViStatus _VI_FUNC rssma_GetROSCAdjustmentActive (ViSession instrumentHandle,
                                                 ViPBoolean adjustmentActive);
ViStatus _VI_FUNC rssma_SetROSCAdjustmentFrequency (ViSession instrumentHandle,
                                                    ViInt32 adjustmentFrequency);
ViStatus _VI_FUNC rssma_GetROSCAdjustmentFrequency (ViSession instrumentHandle,
                                                    ViPInt32 adjustmentFrequency);
ViStatus _VI_FUNC rssma_SetRFDeltaPhase (ViSession instrumentHandle,
                                         ViReal64 deltaPhase);
ViStatus _VI_FUNC rssma_GetRFDeltaPhase (ViSession instrumentHandle,
                                         ViPReal64 deltaPhase);
ViStatus _VI_FUNC rssma_ResetRFDeltaPhaseDisplay (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_confRFLevel (ViSession instrumentHandle, ViReal64 amplitude,
                                     ViReal64 offset, ViReal64 limit,
                                     ViBoolean variationActive,
                                     ViReal64 variationStep, ViInt32 attenuatorMode,
                                     ViInt32 RFLevelOnRecall);
ViStatus _VI_FUNC rssma_SetRFAmplitude (ViSession instrumentHandle,
                                        ViReal64 amplitude);
ViStatus _VI_FUNC rssma_GetRFAmplitude (ViSession instrumentHandle,
                                        ViPReal64 amplitude);

ViStatus _VI_FUNC rssma_SetRFOutputLevel (ViSession instrumentHandle,
                                          ViReal64 RFOutputLevel);

ViStatus _VI_FUNC rssma_GetRFOutputLevel (ViSession instrumentHandle,
                                          ViReal64 *RFOutputLevel);

ViStatus _VI_FUNC rssma_SetRFLevelLimit (ViSession instrumentHandle,
                                         ViReal64 limit);
ViStatus _VI_FUNC rssma_GetRFLevelLimit (ViSession instrumentHandle,
                                         ViPReal64 limit);
ViStatus _VI_FUNC rssma_SetRFLevelOffset (ViSession instrumentHandle,
                                          ViReal64 offset);
ViStatus _VI_FUNC rssma_GetRFLevelOffset (ViSession instrumentHandle,
                                          ViPReal64 offset);
ViStatus _VI_FUNC rssma_SetRFLevelonRecall (ViSession instrumentHandle,
                                            ViInt32 RFLevelOnRecall);
ViStatus _VI_FUNC rssma_GetRFLevelonRecall (ViSession instrumentHandle,
                                            ViPInt32 RFLevelOnRecall);
ViStatus _VI_FUNC rssma_SetRFAttenuatorMode (ViSession instrumentHandle,
                                             ViInt32 attenuatorMode);
ViStatus _VI_FUNC rssma_GetRFAttenuatorMode (ViSession instrumentHandle,
                                             ViPInt32 attenuatorMode);
ViStatus _VI_FUNC rssma_GetRFAttenuatorLowerRange (ViSession instrumentHandle,
                                                   ViPReal64 attenuatorLowerRange);
ViStatus _VI_FUNC rssma_GetRFAttenuatorUpperRange (ViSession instrumentHandle,
                                                   ViPReal64 attenuatorUpperRange);

ViStatus _VI_FUNC rssma_SetRFAttenuatorRFOffMode (ViSession instrumentHandle,
                                                  ViInt32 RFOFFMode);

ViStatus _VI_FUNC rssma_GetRFAttenuatorRFOffMode (ViSession instrumentHandle,
                                                  ViInt32 *RFOFFMode);

ViStatus _VI_FUNC rssma_SetRFLevelVariationActive (ViSession instrumentHandle,
                                                   ViBoolean variationActive);
ViStatus _VI_FUNC rssma_GetRFLevelVariationActive (ViSession instrumentHandle,
                                                   ViPBoolean variationActive);
ViStatus _VI_FUNC rssma_SetRFLevelVariationStep (ViSession instrumentHandle,
                                                 ViReal64 variationStep);
ViStatus _VI_FUNC rssma_GetRFLevelVariationStep (ViSession instrumentHandle,
                                                 ViPReal64 variationStep);

ViStatus _VI_FUNC rssma_confNRPZPowerControl (ViSession instrumentHandle,
                                              ViInt32 sensor, ViBoolean state,
                                              ViReal64 targetLevel,
                                              ViReal64 RFLevelLimit,
                                              ViReal64 catchRange,
                                              ViInt32 delayTime,
                                              ViBoolean usePeakPower);

ViStatus _VI_FUNC rssma_SetPowerControlSensor (ViSession instrumentHandle,
                                               ViInt32 sensor);

ViStatus _VI_FUNC rssma_GetPowerControlSensor (ViSession instrumentHandle,
                                               ViInt32 *sensor);

ViStatus _VI_FUNC rssma_SetPowerControlSensorState (ViSession instrumentHandle,
                                                    ViBoolean state);

ViStatus _VI_FUNC rssma_GetPowerControlSensorState (ViSession instrumentHandle,
                                                    ViBoolean *state);

ViStatus _VI_FUNC rssma_SetPowerControlSensorTargetLevel (ViSession instrumentHandle,
                                                          ViReal64 level);

ViStatus _VI_FUNC rssma_GetPowerControlSensorTargetLevel (ViSession instrumentHandle,
                                                          ViReal64 *level);

ViStatus _VI_FUNC rssma_SetPowerControlSensorCatchRange (ViSession instrumentHandle,
                                                         ViReal64 catchRange);

ViStatus _VI_FUNC rssma_GetPowerControlSensorCatchRange (ViSession instrumentHandle,
                                                         ViReal64 *catchRange);

ViStatus _VI_FUNC rssma_SetPowerControlSensorDelayTime (ViSession instrumentHandle,
                                                        ViInt32 delayTime);

ViStatus _VI_FUNC rssma_GetPowerControlSensorDelayTime (ViSession instrumentHandle,
                                                        ViInt32 *delayTime);

ViStatus _VI_FUNC rssma_SetPowerControlSensorPeakPowerState (ViSession instrumentHandle,
                                                            ViBoolean state);

ViStatus _VI_FUNC rssma_GetPowerControlSensorPeakPowerState (ViSession instrumentHandle,
                                                            ViBoolean *state);

ViStatus _VI_FUNC rssma_SetRFPowerOnState (ViSession instrumentHandle,
                                           ViInt32 powerOnState);
ViStatus _VI_FUNC rssma_GetRFPowerOnState (ViSession instrumentHandle,
                                           ViPInt32 powerOnState);
ViStatus _VI_FUNC rssma_SetALCState (ViSession instrumentHandle, ViInt32 ALCState);
ViStatus _VI_FUNC rssma_GetALCState (ViSession instrumentHandle, ViPInt32 ALCState);
ViStatus _VI_FUNC rssma_ALCSearchOnce (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetUserCorrectionDataState (ViSession instrumentHandle,
                                                    ViBoolean correctionState);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataState (ViSession instrumentHandle,
                                                    ViPBoolean correctionState);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataCatalog (ViSession instrumentHandle,
                                                      ViString directoryPath,
                                                      ViInt32 arraySize,
                                                      ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_SetUserCorrectionDataTable (ViSession instrumentHandle,
                                                    ViString correctionTable);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataTable (ViSession instrumentHandle,
                                                    ViChar _VI_FAR correctionTable[]);
ViStatus _VI_FUNC rssma_GetUserCorrectionValue (ViSession instrumentHandle,
                                                ViPReal64 userCorrectionValue);
ViStatus _VI_FUNC rssma_SetUserCorrectionDataFrequencyValues
             (ViSession instrumentHandle, ViInt32 arraySize,
              ViReal64 _VI_FAR frequencyValues[]);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataFrequencyValues
             (ViSession instrumentHandle, ViInt32 arraySize,
              ViReal64 _VI_FAR frequencyValues[]);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataNumberofFrequencyValues
             (ViSession instrumentHandle, ViPInt32 numberOfFrequencyValues);
ViStatus _VI_FUNC rssma_SetUserCorrectionDataLevelValues (ViSession instrumentHandle,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR levelValues[]);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataLevelValues (ViSession instrumentHandle,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR levelValues[]);
ViStatus _VI_FUNC rssma_GetUserCorrectionDataNumberofLevelValues
             (ViSession instrumentHandle, ViPInt32 numberOfLevelValues);
ViStatus _VI_FUNC rssma_UserCorrectionFillListWithSensor (ViSession instrumentHandle,
                                                          ViInt32 sensor,
                                                          ViInt32 timeout);
ViStatus _VI_FUNC rssma_UserCorrectionDataDeleteTable (ViSession instrumentHandle,
                                                       ViString tableName);
ViStatus _VI_FUNC rssma_SetALCOffMode (ViSession instrumentHandle,
                                       ViInt32 listMode);
ViStatus _VI_FUNC rssma_GetALCOffMode (ViSession instrumentHandle,
                                       ViPInt32 listMode);
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportListMode
             (ViSession instrumentHandle, ViInt32 listMode);
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportListMode
             (ViSession instrumentHandle, ViPInt32 listMode);
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportListFile
             (ViSession instrumentHandle, ViString sourceFile);
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportListFile
             (ViSession instrumentHandle, ViChar _VI_FAR sourceFile[]);
ViStatus _VI_FUNC rssma_UserCorrectionImportExportExecute
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIFile
             (ViSession instrumentHandle, ViString ASCIISourceFile);
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIFile
             (ViSession instrumentHandle, ViChar _VI_FAR ASCIISourceFile[]);
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIExtension
             (ViSession instrumentHandle, ViInt32 extension);
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIExtension
             (ViSession instrumentHandle, ViPInt32 extension);
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIColumnSeparator
             (ViSession instrumentHandle, ViInt32 separator);
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIColumnSeparator
             (ViSession instrumentHandle, ViPInt32 separator);
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIDecimalSeparator
             (ViSession instrumentHandle, ViInt32 decimalSeparator);
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIDecimalSeparator
             (ViSession instrumentHandle, ViPInt32 decimalSeparator);
ViStatus _VI_FUNC rssma_UserCorrectionImportExportCatalog
             (ViSession instrumentHandle, ViInt32 arraySize,
              ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_confPowerSense (ViSession instrumentHandle, ViInt32 sense,
                                        ViBoolean initPowerMeas, ViInt32 unit,
                                        ViBoolean permanentDisplay, ViInt32 source,
                                        ViReal64 frequency, ViInt32 filterMode);

ViStatus _VI_FUNC rssma_confPowerResponseMeasurement (ViSession instrumentHandle,
                                                      ViInt32 sense,
                                                      ViInt32 sweepMode,
                                                      ViReal64 start, ViReal64 stop,
                                                      ViInt32 timing, ViInt32 steps,
                                                      ViInt32 spacing);
ViStatus _VI_FUNC rssma_confPowerResponseMeasurementLevelOffset (ViSession instrumentHandle,
                                                   ViInt32 sense,
                                                   ViInt32 sweepMode,
                                                   ViBoolean offsetState,
                                                   ViReal64 offset);
ViStatus _VI_FUNC rssma_confPowerResponseMeasurementDiagram
             (ViSession instrumentHandle, ViInt32 sense,
              ViReal64 frequencyYScaleMinimum, ViReal64 frequencyYScaleMaximum,
              ViReal64 powerYScaleMinimum, ViReal64 powerYScaleMaximum,
              ViInt32 senseMode);
ViStatus _VI_FUNC rssma_PowerZero (ViSession instrumentHandle, ViInt32 sense);
ViStatus _VI_FUNC rssma_SetInitiatePowerMeasurement (ViSession instrumentHandle,
                                                     ViInt32 sense,
                                                     ViBoolean initPowerMeas);
ViStatus _VI_FUNC rssma_GetInitiatePowerMeasurement (ViSession instrumentHandle,
                                                     ViInt32 sense,
                                                     ViPBoolean initPowerMeas);
ViStatus _VI_FUNC rssma_SetPowerUseSParameters (ViSession instrumentHandle,
                                                ViInt32 sense,
                                                ViBoolean useSParameters);
ViStatus _VI_FUNC rssma_GetPowerUseSParameters (ViSession instrumentHandle,
                                                ViInt32 sense,
                                                ViPBoolean useSParameters);
ViStatus _VI_FUNC rssma_SetPowerFrequency (ViSession instrumentHandle,
                                           ViInt32 sense, ViReal64 frequency);
ViStatus _VI_FUNC rssma_GetPowerFrequency (ViSession instrumentHandle,
                                           ViInt32 sense, ViPReal64 frequency);
ViStatus _VI_FUNC rssma_SetPowerOffset (ViSession instrumentHandle, ViInt32 sense,
                                        ViReal64 offset);
ViStatus _VI_FUNC rssma_GetPowerOffset (ViSession instrumentHandle, ViInt32 sense,
                                        ViPReal64 offset);
ViStatus _VI_FUNC rssma_SetPowerOffsetState (ViSession instrumentHandle,
                                             ViInt32 sense, ViBoolean offsetState);
ViStatus _VI_FUNC rssma_GetPowerOffsetState (ViSession instrumentHandle,
                                             ViInt32 sense, ViPBoolean offsetState);
ViStatus _VI_FUNC rssma_SetPowerSource (ViSession instrumentHandle, ViInt32 sense,
                                        ViInt32 source);
ViStatus _VI_FUNC rssma_GetPowerSource (ViSession instrumentHandle, ViInt32 sense,
                                        ViPInt32 source);
ViStatus _VI_FUNC rssma_SetPowerUnit (ViSession instrumentHandle, ViInt32 sense,
                                      ViInt32 unit);
ViStatus _VI_FUNC rssma_GetPowerUnit (ViSession instrumentHandle, ViInt32 sense,
                                      ViPInt32 unit);
ViStatus _VI_FUNC rssma_SetPowerDisplayPermanentState (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViBoolean permanentDisplay);
ViStatus _VI_FUNC rssma_GetPowerDisplayPermanentState (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViPBoolean permanentDisplay);
ViStatus _VI_FUNC rssma_SetPowerDisplayPermanentPriority (ViSession instrumentHandle,
                                                          ViInt32 trace,
                                                          ViInt32 priority);
ViStatus _VI_FUNC rssma_GetPowerDisplayPermanentPriority (ViSession instrumentHandle,
                                                          ViInt32 trace,
                                                          ViPInt32 priority);
ViStatus _VI_FUNC rssma_confPowerFilter (ViSession instrumentHandle, ViInt32 sense,
                                         ViInt32 filterMode, ViReal64 noiseContent,
                                         ViReal64 timeout);
ViStatus _VI_FUNC rssma_SetPowerFilterLength (ViSession instrumentHandle,
                                              ViInt32 sense, ViInt32 filterLength);
ViStatus _VI_FUNC rssma_GetPowerFilterLength (ViSession instrumentHandle,
                                              ViInt32 sense, ViPInt32 filterLength);
ViStatus _VI_FUNC rssma_SetPowerFilterMode (ViSession instrumentHandle,
                                            ViInt32 sense, ViInt32 filterMode);
ViStatus _VI_FUNC rssma_GetPowerFilterMode (ViSession instrumentHandle,
                                            ViInt32 sense, ViPInt32 filterMode);
ViStatus _VI_FUNC rssma_SetPowerFilterFixedNoiseContent (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViReal64 noiseContent);
ViStatus _VI_FUNC rssma_GetPowerFilterFixedNoiseContent (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViPReal64 noiseContent);
ViStatus _VI_FUNC rssma_SetPowerFilterFixedNoiseTimeout (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViReal64 timeout);
ViStatus _VI_FUNC rssma_GetPowerFilterFixedNoiseTimeout (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViPReal64 timeout);
ViStatus _VI_FUNC rssma_GetPowerFilterLengthAuto (ViSession instrumentHandle,
                                                  ViInt32 sense,
                                                  ViPInt32 filterLengthAuto);
ViStatus _VI_FUNC rssma_PowerFilterSearchOnce (ViSession instrumentHandle,
                                               ViInt32 sense);
ViStatus _VI_FUNC rssma_SetPowerSensorGateState (ViSession instrumentHandle,
                                                 ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSensorGateState (ViSession instrumentHandle,
                                                 ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSensorGateStart (ViSession instrumentHandle,
                                                 ViInt32 gate, ViReal64 start);
ViStatus _VI_FUNC rssma_GetPowerSensorGateStart (ViSession instrumentHandle,
                                                 ViInt32 gate, ViPReal64 start);
ViStatus _VI_FUNC rssma_SetPowerSensorGateStop (ViSession instrumentHandle,
                                                ViInt32 gate, ViReal64 stop);
ViStatus _VI_FUNC rssma_GetPowerSensorGateStop (ViSession instrumentHandle,
                                                ViInt32 gate, ViPReal64 stop);
ViStatus _VI_FUNC rssma_SetPowerSensorGateTrace (ViSession instrumentHandle,
                                                 ViInt32 trace);
ViStatus _VI_FUNC rssma_GetPowerSensorGateTrace (ViSession instrumentHandle,
                                                 ViPInt32 trace);
ViStatus _VI_FUNC rssma_GetPowerSensorGateAveragePower (ViSession instrumentHandle,
                                                        ViInt32 gate,
                                                        ViPReal64 averagePower);
ViStatus _VI_FUNC rssma_GetPowerSensorGatePeakPower (ViSession instrumentHandle,
                                                     ViInt32 gate,
                                                     ViPReal64 peakPower);
ViStatus _VI_FUNC rssma_PowerSweepInitiate (ViSession instrumentHandle,
                                            ViInt32 sense, ViInt32 timeout);
ViStatus _VI_FUNC rssma_PowerSweepAbort (ViSession instrumentHandle, ViInt32 sense);
ViStatus _VI_FUNC rssma_confPowerSweepFrequencySeparateFrequencyRange
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state,
              ViReal64 separateFrequencyStart, ViReal64 separateFrequencyStop);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyStart (ViSession instrumentHandle,
                                                     ViInt32 sense,
                                                     ViReal64 frequencyStart);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyStart (ViSession instrumentHandle,
                                                     ViInt32 sense,
                                                     ViPReal64 frequencyStart);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyStop (ViSession instrumentHandle,
                                                    ViInt32 sense,
                                                    ViReal64 frequencyStop);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyStop (ViSession instrumentHandle,
                                                    ViInt32 sense,
                                                    ViPReal64 frequencyStop);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyTiming (ViSession instrumentHandle,
                                                      ViInt32 sense,
                                                      ViInt32 frequencyTiming);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyTiming (ViSession instrumentHandle,
                                                      ViInt32 sense,
                                                      ViPInt32 frequencyTiming);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySpacing (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViInt32 frequencySpacing);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySpacing (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViPInt32 frequencySpacing);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySteps (ViSession instrumentHandle,
                                                     ViInt32 sense,
                                                     ViInt32 frequencySteps);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySteps (ViSession instrumentHandle,
                                                     ViInt32 sense,
                                                     ViPInt32 frequencySteps);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyYScaleMinimum
             (ViSession instrumentHandle, ViInt32 sense, ViReal64 yScaleMinimum);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyYScaleMinimum
             (ViSession instrumentHandle, ViInt32 sense, ViPReal64 yScaleMinimum);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyYScaleMaximum
             (ViSession instrumentHandle, ViInt32 sense, ViReal64 yScaleMaximum);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyYScaleMaximum
             (ViSession instrumentHandle, ViInt32 sense, ViPReal64 yScaleMaximum);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyExecution (ViSession instrumentHandle,
                                                         ViInt32 execution);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyExecution (ViSession instrumentHandle,
                                                         ViPInt32 execution);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyYAutoScale (ViSession instrumentHandle,
                                                          ViInt32 autoScale);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyYAutoScale (ViSession instrumentHandle,
                                                          ViPInt32 autoScale);
ViStatus _VI_FUNC rssma_ResetPowerSweepFrequencyYScale (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySeparateFrequencyRange
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySeparateFrequencyRange
             (ViSession instrumentHandle, ViInt32 sense, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySeparateFrequencyStart
             (ViSession instrumentHandle, ViInt32 sense,
              ViReal64 separateFrequencyStart);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySeparateFrequencyStart
             (ViSession instrumentHandle, ViInt32 sense,
              ViPReal64 separateFrequencyStart);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySeparateFrequencyStop
             (ViSession instrumentHandle, ViInt32 sense,
              ViReal64 separateFrequencyStop);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySeparateFrequencyStop
             (ViSession instrumentHandle, ViInt32 sense,
              ViPReal64 separateFrequencyStop);
ViStatus _VI_FUNC rssma_PowerSweepFrequencyReferenceCurve
             (ViSession instrumentHandle, ViInt32 sense);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyReferenceCurvePoints
             (ViSession instrumentHandle, ViInt32 sense, ViPInt32 points);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyReferenceCurveFrequencyX (ViSession instrumentHandle,
                                                                        ViInt32 sense,
                                                                        ViInt32 arraySize,
                                                                        ViReal64 frequencyX[]);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyReferenceCurveFrequencyX (ViSession instrumentHandle,
                                                                        ViInt32 sense,
                                                                        ViInt32 arraySize,
                                                                        ViReal64 frequencyX[]);
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyReferenceCurvePowerY (ViSession instrumentHandle,
                                                                    ViInt32 sense,
                                                                    ViInt32 arraySize,
                                                                    ViReal64 powerY[]);
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyReferenceCurvePowerY (ViSession instrumentHandle,
                                                                    ViInt32 sense,
                                                                    ViInt32 arraySize,
                                                                    ViReal64 powerY[]);

ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyLevelOffsetState (ViSession instrumentHandle,
                                                                ViInt32 sense,
                                                                ViBoolean offsetState);

ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyLevelOffsetState (ViSession instrumentHandle,
                                                                ViInt32 sense,
                                                                ViBoolean *offsetState);

ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyLevelOffset (ViSession instrumentHandle,
                                                           ViInt32 sense,
                                                           ViReal64 offset);

ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyLevelOffset (ViSession instrumentHandle,
                                                           ViInt32 sense,
                                                           ViReal64 *offset);

ViStatus _VI_FUNC rssma_confPowerSweepPowerSeparateFrequencyRange
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state,
              ViReal64 separateFrequency);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerStart (ViSession instrumentHandle,
                                                 ViInt32 sense,
                                                 ViReal64 powerStart);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerStart (ViSession instrumentHandle,
                                                 ViInt32 sense,
                                                 ViPReal64 powerStart);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerStop (ViSession instrumentHandle,
                                                ViInt32 sense, ViReal64 powerStop);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerStop (ViSession instrumentHandle,
                                                ViInt32 sense, ViPReal64 powerStop);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSpacing (ViSession instrumentHandle,
                                                   ViInt32 sense,
                                                   ViInt32 powerSpacing);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSpacing (ViSession instrumentHandle,
                                                   ViInt32 sense,
                                                   ViPInt32 powerSpacing);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerTiming (ViSession instrumentHandle,
                                                  ViInt32 sense,
                                                  ViInt32 powerTiming);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerTiming (ViSession instrumentHandle,
                                                  ViInt32 sense,
                                                  ViPInt32 powerTiming);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSteps (ViSession instrumentHandle,
                                                 ViInt32 sense, ViInt32 powerSteps);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSteps (ViSession instrumentHandle,
                                                 ViInt32 sense,
                                                 ViPInt32 powerSteps);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerYScaleMinimum (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViReal64 yScaleMinimum);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerYScaleMinimum (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViPReal64 yScaleMinimum);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerYScaleMaximum (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViReal64 yScaleMaximum);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerYScaleMaximum (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViPReal64 yScaleMaximum);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerExecution (ViSession instrumentHandle,
                                                     ViInt32 execution);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerExecution (ViSession instrumentHandle,
                                                     ViPInt32 execution);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerYAutoScale (ViSession instrumentHandle,
                                                      ViInt32 autoScale);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerYAutoScale (ViSession instrumentHandle,
                                                      ViPInt32 autoScale);
ViStatus _VI_FUNC rssma_ResetPowerSweepPowerYScale (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSeparateFrequencyState
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSeparateFrequencyState
             (ViSession instrumentHandle, ViInt32 sense, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSeparateFrequency
             (ViSession instrumentHandle, ViInt32 sense,
              ViReal64 separateFrequency);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSeparateFrequency
             (ViSession instrumentHandle, ViInt32 sense,
              ViPReal64 separateFrequency);
ViStatus _VI_FUNC rssma_PowerSweepPowerReferenceCurve (ViSession instrumentHandle,
                                                       ViInt32 sense);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerReferenceCurvePoints
             (ViSession instrumentHandle, ViInt32 sense, ViPInt32 points);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerReferenceCurvePowerX (ViSession instrumentHandle,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerX[]);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerReferenceCurvePowerX (ViSession instrumentHandle,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerX[]);
ViStatus _VI_FUNC rssma_SetPowerSweepPowerReferenceCurvePowerY (ViSession instrumentHandle,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerY[]);
ViStatus _VI_FUNC rssma_GetPowerSweepPowerReferenceCurvePowerY (ViSession instrumentHandle,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerY[]);

ViStatus _VI_FUNC rssma_SetPowerSweepPowerLevelOffsetState (ViSession instrumentHandle,
                                                            ViInt32 sense,
                                                            ViBoolean offsetState);

ViStatus _VI_FUNC rssma_GetPowerSweepPowerLevelOffsetState (ViSession instrumentHandle,
                                                            ViInt32 sense,
                                                            ViBoolean *offsetState);

ViStatus _VI_FUNC rssma_SetPowerSweepPowerLevelOffset (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViReal64 offset);

ViStatus _VI_FUNC rssma_GetPowerSweepPowerLevelOffset (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViReal64 *offset);

ViStatus _VI_FUNC rssma_confPowerSweepTimeSeparateFrequencyRange
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state,
              ViReal64 separateFrequency);
ViStatus _VI_FUNC rssma_SetPowerSweepTimePulseDataAnalysisState
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepTimePulseDataAnalysisState
             (ViSession instrumentHandle, ViInt32 sense, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSweepTimePulseThresholdBase
             (ViSession instrumentHandle, ViInt32 sense, ViInt32 base);
ViStatus _VI_FUNC rssma_GetPowerSweepTimePulseThresholdBase
             (ViSession instrumentHandle, ViInt32 sense, ViPInt32 base);
ViStatus _VI_FUNC rssma_SetPowerSweepTimePulseThresholdReferenceLevel
             (ViSession instrumentHandle, ViInt32 sense, ViInt32 mode,
              ViReal64 referenceLevel);
ViStatus _VI_FUNC rssma_GetPowerSweepTimePulseThresholdReferenceLevel
             (ViSession instrumentHandle, ViInt32 sense, ViInt32 mode,
              ViPReal64 referenceLevel);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerSource (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViInt32 source);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerSource (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViPInt32 source);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerDropOutTime
             (ViSession instrumentHandle, ViInt32 sense, ViReal64 dropOutTime);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerDropOutTime
             (ViSession instrumentHandle, ViInt32 sense, ViPReal64 dropOutTime);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerHysteresis
             (ViSession instrumentHandle, ViInt32 sense, ViReal64 hysteresis);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerHysteresis
             (ViSession instrumentHandle, ViInt32 sense, ViPReal64 hysteresis);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerThreshold
             (ViSession instrumentHandle, ViInt32 sense, ViReal64 threshold);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerThreshold
             (ViSession instrumentHandle, ViInt32 sense, ViPReal64 threshold);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerEvents (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViInt32 events);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerEvents (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViPInt32 events);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerAutoSet (ViSession instrumentHandle,
                                                         ViInt32 sense,
                                                         ViInt32 timeout);

ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerSlope (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViInt32 slope);

ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerSlope (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViInt32 *slope);

ViStatus _VI_FUNC rssma_SetPowerSweepTimeStart (ViSession instrumentHandle,
                                                ViInt32 sense, ViReal64 timeStart);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeStart (ViSession instrumentHandle,
                                                ViInt32 sense, ViPReal64 timeStart);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeStop (ViSession instrumentHandle,
                                               ViInt32 sense, ViReal64 timeStop);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeStop (ViSession instrumentHandle,
                                               ViInt32 sense, ViPReal64 timeStop);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerEvent (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViInt32 triggerEvents);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerEvent (ViSession instrumentHandle,
                                                       ViInt32 sense,
                                                       ViPInt32 triggerEvents);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSteps (ViSession instrumentHandle,
                                                ViInt32 sense, ViInt32 timeSteps);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSteps (ViSession instrumentHandle,
                                                ViInt32 sense, ViPInt32 timeSteps);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSpacing (ViSession instrumentHandle,
                                                  ViInt32 sense,
                                                  ViInt32 powerSpacing);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSpacing (ViSession instrumentHandle,
                                                  ViInt32 sense,
                                                  ViPInt32 powerSpacing);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeAveragingFactor (ViSession instrumentHandle,
                                                          ViInt32 averagingFactor);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeAveragingFactor (ViSession instrumentHandle,
                                                          ViPInt32 averagingFactor);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeYScaleMinimum (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViReal64 yScaleMinimum);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeYScaleMinimum (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViPReal64 yScaleMinimum);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeYScaleMaximum (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViReal64 yScaleMaximum);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeYScaleMaximum (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViPReal64 yScaleMaximum);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeExecution (ViSession instrumentHandle,
                                                    ViInt32 execution);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeExecution (ViSession instrumentHandle,
                                                    ViPInt32 execution);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeYAutoScale (ViSession instrumentHandle,
                                                     ViInt32 autoScale);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeYAutoScale (ViSession instrumentHandle,
                                                     ViPInt32 autoScale);
ViStatus _VI_FUNC rssma_ResetPowerSweepTimeYScale (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSeparateFrequencyState
             (ViSession instrumentHandle, ViInt32 sense, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSeparateFrequencyState
             (ViSession instrumentHandle, ViInt32 sense, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSeparateFrequency
             (ViSession instrumentHandle, ViInt32 sense,
              ViReal64 separateFrequency);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSeparateFrequency
             (ViSession instrumentHandle, ViInt32 sense,
              ViPReal64 separateFrequency);
ViStatus _VI_FUNC rssma_PowerSweepTimeReferenceCurve (ViSession instrumentHandle,
                                                      ViInt32 sense);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeReferenceCurvePoints
             (ViSession instrumentHandle, ViInt32 sense, ViPInt32 points);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeReferenceCurveTimeX (ViSession instrumentHandle,
                                                              ViInt32 sense,
                                                              ViInt32 arraySize,
                                                              ViReal64 timeX[]);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeReferenceCurveTimeX (ViSession instrumentHandle,
                                                              ViInt32 sense,
                                                              ViInt32 arraySize,
                                                              ViReal64 timeX[]);
ViStatus _VI_FUNC rssma_SetPowerSweepTimeReferenceCurvePowerY (ViSession instrumentHandle,
                                                               ViInt32 sense,
                                                               ViInt32 arraySize,
                                                               ViReal64 powerY[]);
ViStatus _VI_FUNC rssma_GetPowerSweepTimeReferenceCurvePowerY (ViSession instrumentHandle,
                                                               ViInt32 sense,
                                                               ViInt32 arraySize,
                                                               ViReal64 powerY[]);

ViStatus _VI_FUNC rssma_SetPowerSweepTimeLevelOffsetState (ViSession instrumentHandle,
                                                           ViInt32 sense,
                                                           ViBoolean offsetState);

ViStatus _VI_FUNC rssma_GetPowerSweepTimeLevelOffsetState (ViSession instrumentHandle,
                                                           ViInt32 sense,
                                                           ViBoolean *offsetState);

ViStatus _VI_FUNC rssma_SetPowerSweepTimeLevelOffset (ViSession instrumentHandle,
                                                      ViInt32 sense,
                                                      ViReal64 offset);

ViStatus _VI_FUNC rssma_GetPowerSweepTimeLevelOffset (ViSession instrumentHandle,
                                                      ViInt32 sense,
                                                      ViReal64 *offset);

ViStatus _VI_FUNC rssma_SetPowerSweepTraceState (ViSession instrumentHandle,
                                                 ViInt32 trace, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepTraceState (ViSession instrumentHandle,
                                                 ViInt32 trace, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSweepTraceColor (ViSession instrumentHandle,
                                                 ViInt32 trace, ViInt32 color);
ViStatus _VI_FUNC rssma_GetPowerSweepTraceColor (ViSession instrumentHandle,
                                                 ViInt32 trace, ViPInt32 color);
ViStatus _VI_FUNC rssma_copyPowerSweepTrace (ViSession instrumentHandle,
                                             ViInt32 trace, ViInt32 copyMode);
ViStatus _VI_FUNC rssma_GetPowerSweepTraceDataPoints (ViSession instrumentHandle,
                                                      ViInt32 trace,
                                                      ViPInt32 dataPoints);
ViStatus _VI_FUNC rssma_GetPowerSweepTraceXValues (ViSession instrumentHandle,
                                                   ViInt32 trace, ViInt32 arraySize,
                                                   ViReal64 _VI_FAR xValues[]);
ViStatus _VI_FUNC rssma_GetPowerSweepTraceYValues (ViSession instrumentHandle,
                                                   ViInt32 trace, ViInt32 arraySize,
                                                   ViReal64 _VI_FAR yValues[]);
ViStatus _VI_FUNC rssma_SetPowerSweepTraceSource (ViSession instrumentHandle,
                                                  ViInt32 trace, ViInt32 source);
ViStatus _VI_FUNC rssma_GetPowerSweepTraceSource (ViSession instrumentHandle,
                                                  ViInt32 trace, ViPInt32 source);
ViStatus _VI_FUNC rssma_GetPowerSweepTracePulseDataAnalysisState
             (ViSession instrumentHandle, ViInt32 trace, ViPBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepTracePulseThresholdBase
             (ViSession instrumentHandle, ViInt32 trace, ViPInt32 base);
ViStatus _VI_FUNC rssma_GetPowerSweepTracePulseThresholdReferenceLevel
             (ViSession instrumentHandle, ViInt32 trace, ViInt32 mode,
              ViPReal64 referenceLevel);
ViStatus _VI_FUNC rssma_SetPowerSweepSenseMode (ViSession instrumentHandle,
                                                ViInt32 sense, ViInt32 senseMode);
ViStatus _VI_FUNC rssma_GetPowerSweepSenseMode (ViSession instrumentHandle,
                                                ViInt32 sense, ViPInt32 senseMode);
ViStatus _VI_FUNC rssma_SetPowerSweepMode (ViSession instrumentHandle,
                                           ViInt32 sense, ViInt32 sweepMode);
ViStatus _VI_FUNC rssma_GetPowerSweepMode (ViSession instrumentHandle,
                                           ViInt32 sense, ViPInt32 sweepMode);
ViStatus _VI_FUNC rssma_SetPowerDisplayIndication (ViSession instrumentHandle,
                                                   ViInt32 trace, ViInt32 type,
                                                   ViBoolean indication);
ViStatus _VI_FUNC rssma_GetPowerDisplayIndication (ViSession instrumentHandle,
                                                   ViInt32 trace, ViInt32 type,
                                                   ViPBoolean indication);
ViStatus _VI_FUNC rssma_GetPowerDisplayPulseDataAnalysis (ViSession instrumentHandle,
                                                          ViInt32 trace,
                                                          ViInt32 type,
                                                          ViPReal64 pulseDataAnalysis);
ViStatus _VI_FUNC rssma_PowerSensorDevice (ViSession instrumentHandle,
                                           ViInt32 sense, ViPInt32 device);
ViStatus _VI_FUNC rssma_PowerSensorType (ViSession instrumentHandle, ViInt32 sense,
                                         ViChar _VI_FAR type[]);
ViStatus _VI_FUNC rssma_PowerReadPower (ViSession instrumentHandle, ViInt32 read,
                                        ViPReal64 power);
ViStatus _VI_FUNC rssma_GetPowerSensorVersion (ViSession instrumentHandle,
                                               ViInt32 sense,
                                               ViChar _VI_FAR version[]);
ViStatus _VI_FUNC rssma_PowerSensorSerialNumber (ViSession instrumentHandle,
                                                 ViInt32 sense,
                                                 ViChar _VI_FAR serialNumber[]);
ViStatus _VI_FUNC rssma_confDiagramDisplay (ViSession instrumentHandle,
                                            ViBoolean grid,
                                            ViInt32 backgroundColor);
ViStatus _VI_FUNC rssma_SetDiagramGrid (ViSession instrumentHandle, ViBoolean grid);
ViStatus _VI_FUNC rssma_GetDiagramGrid (ViSession instrumentHandle,
                                        ViPBoolean grid);
ViStatus _VI_FUNC rssma_SetDiagramBackgroundColor (ViSession instrumentHandle,
                                                   ViInt32 backgroundColor);
ViStatus _VI_FUNC rssma_GetDiagramBackgroundColor (ViSession instrumentHandle,
                                                   ViPInt32 backgroundColor);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyLNGFormat (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViInt32 sweepHardcopyFormat);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyLNGFormat (ViSession instrumentHandle,
                                                        ViInt32 sense,
                                                        ViPInt32 sweepHardcopyFormat);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyFile (ViSession instrumentHandle,
                                                   ViString fileName);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyFile (ViSession instrumentHandle,
                                                   ViChar _VI_FAR fileName[]);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyDevice (ViSession instrumentHandle,
                                                     ViInt32 hardcopyDevice);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyDevice (ViSession instrumentHandle,
                                                     ViPInt32 hardcopyDevice);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyImageSize (ViSession instrumentHandle,
                                                        ViInt32 imageSize);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyImageSize (ViSession instrumentHandle,
                                                        ViPInt32 imageSize);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyMarkerState (ViSession instrumentHandle,
                                                          ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyMarkerState (ViSession instrumentHandle,
                                                          ViPBoolean state);
ViStatus _VI_FUNC rssma_SavePowerSweepHardcopy (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVHeader (ViSession instrumentHandle,
                                                        ViInt32 CSVHeader);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVHeader (ViSession instrumentHandle,
                                                        ViPInt32 CSVHeader);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVOrientation
             (ViSession instrumentHandle, ViInt32 CSVOrientation);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVOrientation
             (ViSession instrumentHandle, ViPInt32 CSVOrientation);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVColumnSeparator
             (ViSession instrumentHandle, ViInt32 CSVColumnSeparator);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVColumnSeparator
             (ViSession instrumentHandle, ViPInt32 CSVColumnSeparator);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVDecimalPoint
             (ViSession instrumentHandle, ViInt32 CSVDecimalPoint);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVDecimalPoint
             (ViSession instrumentHandle, ViPInt32 CSVDecimalPoint);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVData (ViSession instrumentHandle,
                                                      ViString destination);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticNamingState
             (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticNamingState
             (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileName
             (ViSession instrumentHandle, ViChar _VI_FAR fileName[]);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticDirectory
             (ViSession instrumentHandle, ViString directoryName);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticDirectory
             (ViSession instrumentHandle, ViChar _VI_FAR directoryName[]);
ViStatus _VI_FUNC rssma_DeletePowerSweepHardcopyImageFiles
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticFileDateState
             (ViSession instrumentHandle, ViInt32 fileNamePart, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileDateState
             (ViSession instrumentHandle, ViInt32 fileNamePart, ViPBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileDate
             (ViSession instrumentHandle, ViInt32 fileNamePart,
              ViChar _VI_FAR part[]);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticFilePrefixState
             (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFilePrefixState
             (ViSession instrumentHandle, ViPBoolean state);
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticFilePrefix
             (ViSession instrumentHandle, ViString prefix);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFilePrefix
             (ViSession instrumentHandle, ViChar _VI_FAR prefix[]);
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileNumber
             (ViSession instrumentHandle, ViPInt32 number);
ViStatus _VI_FUNC rssma_SetPowerMathFrequencyState (ViSession instrumentHandle,
                                                    ViInt32 trace, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerMathFrequencyState (ViSession instrumentHandle,
                                                    ViInt32 trace,
                                                    ViPBoolean state);
ViStatus _VI_FUNC rssma_PowerMathFrequencySubtract (ViSession instrumentHandle,
                                                    ViInt32 trace,
                                                    ViInt32 subtract);
ViStatus _VI_FUNC rssma_SetPowerMathPowerState (ViSession instrumentHandle,
                                                ViInt32 trace, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerMathPowerState (ViSession instrumentHandle,
                                                ViInt32 trace, ViPBoolean state);
ViStatus _VI_FUNC rssma_PowerMathPowerSubtract (ViSession instrumentHandle,
                                                ViInt32 trace, ViInt32 subtract);
ViStatus _VI_FUNC rssma_SetPowerMathTimeState (ViSession instrumentHandle,
                                               ViInt32 trace, ViBoolean state);
ViStatus _VI_FUNC rssma_GetPowerMathTimeState (ViSession instrumentHandle,
                                               ViInt32 trace, ViPBoolean state);
ViStatus _VI_FUNC rssma_PowerMathTimeSubtract (ViSession instrumentHandle,
                                               ViInt32 trace, ViInt32 subtract);
ViStatus _VI_FUNC rssma_confFrequencySweep (ViSession instrumentHandle,
                                            ViInt32 mode, ViInt32 frequencySetting,
                                            ViReal64 centerFrequency, ViReal64 span,
                                            ViReal64 startFrequency,
                                            ViReal64 stopFrequency, ViInt32 spacing,
                                            ViReal64 step, ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_FrequencySweepExecute (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetFrequencySweepMode (ViSession instrumentHandle,
                                               ViInt32 mode);
ViStatus _VI_FUNC rssma_SetFrequencySweepStartFreq (ViSession instrumentHandle,
                                                    ViReal64 startFrequency);
ViStatus _VI_FUNC rssma_GetFrequencySweepStartFreq (ViSession instrumentHandle,
                                                    ViPReal64 startFrequency);
ViStatus _VI_FUNC rssma_SetFrequencySweepStopFreq (ViSession instrumentHandle,
                                                   ViReal64 stopFrequency);
ViStatus _VI_FUNC rssma_GetFrequencySweepStopFreq (ViSession instrumentHandle,
                                                   ViPReal64 stopFrequency);
ViStatus _VI_FUNC rssma_SetFrequencySweepCenterFreq (ViSession instrumentHandle,
                                                     ViReal64 centerFrequency);
ViStatus _VI_FUNC rssma_GetFrequencySweepCenterFreq (ViSession instrumentHandle,
                                                     ViPReal64 centerFrequency);
ViStatus _VI_FUNC rssma_SetFrequencySweepSpan (ViSession instrumentHandle,
                                               ViReal64 span);
ViStatus _VI_FUNC rssma_GetFrequencySweepSpan (ViSession instrumentHandle,
                                               ViPReal64 span);
ViStatus _VI_FUNC rssma_SetFrequencySweepSpacing (ViSession instrumentHandle,
                                                  ViInt32 spacing);
ViStatus _VI_FUNC rssma_GetFrequencySweepSpacing (ViSession instrumentHandle,
                                                  ViPInt32 spacing);
ViStatus _VI_FUNC rssma_SetFrequencySweepStepLin (ViSession instrumentHandle,
                                                  ViReal64 stepLin);
ViStatus _VI_FUNC rssma_GetFrequencySweepStepLin (ViSession instrumentHandle,
                                                  ViPReal64 stepLin);
ViStatus _VI_FUNC rssma_SetFrequencySweepStepLog (ViSession instrumentHandle,
                                                  ViReal64 stepLog);
ViStatus _VI_FUNC rssma_GetFrequencySweepStepLog (ViSession instrumentHandle,
                                                  ViPReal64 stepLog);
ViStatus _VI_FUNC rssma_SetFrequencySweepDwellTime (ViSession instrumentHandle,
                                                    ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_GetFrequencySweepDwellTime (ViSession instrumentHandle,
                                                    ViPReal64 dwellTime);
ViStatus _VI_FUNC rssma_SetFrequencySweepManualStep (ViSession instrumentHandle,
                                                     ViReal64 manualStep);
ViStatus _VI_FUNC rssma_GetFrequencySweepManualStep (ViSession instrumentHandle,
                                                     ViPReal64 manualStep);
ViStatus _VI_FUNC rssma_SetFrequencySweepPoints (ViSession instrumentHandle,
                                                 ViInt32 frequencySweepPoints);
ViStatus _VI_FUNC rssma_GetFrequencySweepPoints (ViSession instrumentHandle,
                                                 ViPInt32 frequencySweepPoints);
ViStatus _VI_FUNC rssma_SetFrequencySweepShape (ViSession instrumentHandle,
                                                ViInt32 shape);
ViStatus _VI_FUNC rssma_GetFrequencySweepShape (ViSession instrumentHandle,
                                                ViPInt32 shape);
ViStatus _VI_FUNC rssma_confRFLevelSweep (ViSession instrumentHandle,
                                          ViInt32 levelSweepMode,
                                          ViReal64 startLevel, ViReal64 stopLevel,
                                          ViReal64 stepLin, ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_LevelSweepExecute (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetLevelSweepMode (ViSession instrumentHandle,
                                           ViInt32 mode);
ViStatus _VI_FUNC rssma_SetLevelSweepStartLevel (ViSession instrumentHandle,
                                                 ViReal64 startLevel);
ViStatus _VI_FUNC rssma_GetLevelSweepStartLevel (ViSession instrumentHandle,
                                                 ViPReal64 startLevel);
ViStatus _VI_FUNC rssma_SetLevelSweepStopLevel (ViSession instrumentHandle,
                                                ViReal64 stopLevel);
ViStatus _VI_FUNC rssma_GetLevelSweepStopLevel (ViSession instrumentHandle,
                                                ViPReal64 stopLevel);
ViStatus _VI_FUNC rssma_SetLevelSweepStep (ViSession instrumentHandle,
                                           ViReal64 step);
ViStatus _VI_FUNC rssma_GetLevelSweepStep (ViSession instrumentHandle,
                                           ViPReal64 step);
ViStatus _VI_FUNC rssma_SetLevelSweepDwell (ViSession instrumentHandle,
                                            ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_GetLevelSweepDwell (ViSession instrumentHandle,
                                            ViPReal64 dwellTime);
ViStatus _VI_FUNC rssma_SetLevelSweepManualStep (ViSession instrumentHandle,
                                                 ViReal64 manualStep);
ViStatus _VI_FUNC rssma_GetLevelSweepManualStep (ViSession instrumentHandle,
                                                 ViPReal64 manualStep);
ViStatus _VI_FUNC rssma_SetLevelSweepPoints (ViSession instrumentHandle,
                                             ViInt32 levelSweepPoints);
ViStatus _VI_FUNC rssma_GetLevelSweepPoints (ViSession instrumentHandle,
                                             ViPInt32 levelSweepPoints);
ViStatus _VI_FUNC rssma_SetLevelSweepAttenuatorMode (ViSession instrumentHandle,
                                                     ViInt32 attenuatorMode);
ViStatus _VI_FUNC rssma_GetLevelSweepAttenuatorMode (ViSession instrumentHandle,
                                                     ViPInt32 attenuatorMode);
ViStatus _VI_FUNC rssma_SetLevelSweepShape (ViSession instrumentHandle,
                                            ViInt32 shape);
ViStatus _VI_FUNC rssma_GetLevelSweepShape (ViSession instrumentHandle,
                                            ViPInt32 shape);
ViStatus _VI_FUNC rssma_GetLevelSweepSpacing (ViSession instrumentHandle,
                                              ViPInt32 spacing);
ViStatus _VI_FUNC rssma_ResetLevelSweep (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_confList (ViSession instrumentHandle, ViString listName,
                                  ViInt32 listMode, ViInt32 arraySize,
                                  ViReal64 _VI_FAR frequencyValues[],
                                  ViReal64 _VI_FAR powerValues[],
                                  ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_LISTCatalog (ViSession instrumentHandle,
                                     ViString directoryPath, ViInt32 arraySize,
                                     ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_LISTDeleteList (ViSession instrumentHandle,
                                        ViString listName);
ViStatus _VI_FUNC rssma_LISTDeleteAll (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_LISTResetList (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_LISTTriggerExecute (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetLISTMode (ViSession instrumentHandle, ViInt32 listMode);
ViStatus _VI_FUNC rssma_SetLISTDwellTime (ViSession instrumentHandle,
                                          ViReal64 dwellTime);
ViStatus _VI_FUNC rssma_GetLISTDwellTime (ViSession instrumentHandle,
                                          ViPReal64 dwellTime);
ViStatus _VI_FUNC rssma_SetLISTSelectList (ViSession instrumentHandle,
                                           ViString listName);
ViStatus _VI_FUNC rssma_GetLISTSelectList (ViSession instrumentHandle,
                                           ViChar _VI_FAR listName[]);
ViStatus _VI_FUNC rssma_SetLISTFrequencyValues (ViSession instrumentHandle,
                                                ViInt32 arraySize,
                                                ViReal64 _VI_FAR frequencyValues[]);
ViStatus _VI_FUNC rssma_GetLISTFrequencyValues (ViSession instrumentHandle,
                                                ViInt32 arraySize,
                                                ViReal64 _VI_FAR frequencyValues[]);
ViStatus _VI_FUNC rssma_GetLISTFrequencyPoints (ViSession instrumentHandle,
                                                ViPInt32 numberOfFrequencyPoints);
ViStatus _VI_FUNC rssma_SetLISTPowerValues (ViSession instrumentHandle,
                                            ViInt32 arraySize,
                                            ViReal64 _VI_FAR powerValues[]);
ViStatus _VI_FUNC rssma_GetLISTPowerValues (ViSession instrumentHandle,
                                            ViInt32 arraySize,
                                            ViReal64 _VI_FAR powerValues[]);
ViStatus _VI_FUNC rssma_GetLISTPowerPoints (ViSession instrumentHandle,
                                            ViPInt32 numberOfPowerPoints);
ViStatus _VI_FUNC rssma_SetLISTAttenuatorMode (ViSession instrumentHandle,
                                               ViInt32 attenuatorMode);
ViStatus _VI_FUNC rssma_GetLISTAttenuatorMode (ViSession instrumentHandle,
                                               ViPInt32 attenuatorMode);
ViStatus _VI_FUNC rssma_SetLISTStepIndex (ViSession instrumentHandle,
                                          ViInt32 stepIndex);
ViStatus _VI_FUNC rssma_GetLISTStepIndex (ViSession instrumentHandle,
                                          ViPInt32 stepIndex);
ViStatus _VI_FUNC rssma_SetLISTStartIndex (ViSession instrumentHandle,
                                           ViInt32 startIndex);
ViStatus _VI_FUNC rssma_GetLISTStartIndex (ViSession instrumentHandle,
                                           ViPInt32 startIndex);
ViStatus _VI_FUNC rssma_SetLISTStopIndex (ViSession instrumentHandle,
                                          ViInt32 stopIndex);
ViStatus _VI_FUNC rssma_GetLISTStopIndex (ViSession instrumentHandle,
                                          ViPInt32 stopIndex);
ViStatus _VI_FUNC rssma_LearnLISTModeData (ViSession instrumentHandle);

ViStatus _VI_FUNC rssma_GetLISTFree (ViSession instrumentHandle,
                                     ViInt32 *freeSpace);

ViStatus _VI_FUNC rssma_SetLISTImportExportListMode (ViSession instrumentHandle,
                                                     ViInt32 listMode);
ViStatus _VI_FUNC rssma_GetLISTImportExportListMode (ViSession instrumentHandle,
                                                     ViPInt32 listMode);
ViStatus _VI_FUNC rssma_SetLISTImportExportListFile (ViSession instrumentHandle,
                                                     ViString sourceFile);
ViStatus _VI_FUNC rssma_GetLISTImportExportListFile (ViSession instrumentHandle,
                                                     ViChar _VI_FAR sourceFile[]);
ViStatus _VI_FUNC rssma_ImportExportExecute (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIFile (ViSession instrumentHandle,
                                                      ViString ASCIISourceFile);
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIFile (ViSession instrumentHandle,
                                                      ViChar _VI_FAR ASCIISourceFile[]);
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIExtension
             (ViSession instrumentHandle, ViInt32 extension);
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIExtension
             (ViSession instrumentHandle, ViPInt32 extension);
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIColumnSeparator
             (ViSession instrumentHandle, ViInt32 separator);
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIColumnSeparator
             (ViSession instrumentHandle, ViPInt32 separator);
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIColumnDecimalSeparator
             (ViSession instrumentHandle, ViInt32 decimalSeparator);
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIColumnDecimalSeparator
             (ViSession instrumentHandle, ViPInt32 decimalSeparator);
ViStatus _VI_FUNC rssma_LISTImportExportCatalog (ViSession instrumentHandle,
                                                 ViInt32 arraySize,
                                                 ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_ConfigureClockSynthesis (ViSession instrumentHandle,
                                                 ViBoolean CSYNState,
                                                 ViReal64 frequency,
                                                 ViBoolean variationActive,
                                                 ViReal64 variationStep);
ViStatus _VI_FUNC rssma_SetCSYNState (ViSession instrumentHandle,
                                      ViBoolean CSYNState);
ViStatus _VI_FUNC rssma_GetCSYNState (ViSession instrumentHandle,
                                      ViPBoolean CSYNState);
ViStatus _VI_FUNC rssma_SetCSYNFrequency (ViSession instrumentHandle,
                                          ViReal64 frequency);
ViStatus _VI_FUNC rssma_GetCSYNFrequency (ViSession instrumentHandle,
                                          ViPReal64 frequency);
ViStatus _VI_FUNC rssma_SetCSYNVariationStep (ViSession instrumentHandle,
                                              ViReal64 variationStep);
ViStatus _VI_FUNC rssma_GetCSYNVariationStep (ViSession instrumentHandle,
                                              ViPReal64 variationStep);
ViStatus _VI_FUNC rssma_SetCSYNVariationActive (ViSession instrumentHandle,
                                                ViBoolean variationActive);
ViStatus _VI_FUNC rssma_GetCSYNVariationActive (ViSession instrumentHandle,
                                                ViPBoolean variationActive);
ViStatus _VI_FUNC rssma_SetCSYNPhase (ViSession instrumentHandle, ViReal64 phase);
ViStatus _VI_FUNC rssma_GetCSYNPhase (ViSession instrumentHandle, ViPReal64 phase);
ViStatus _VI_FUNC rssma_SetCSYNPhaseReference (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetCSYNDCOffsetState (ViSession instrumentHandle,
                                              ViBoolean DCOffsetState);
ViStatus _VI_FUNC rssma_GetCSYNDCOffsetState (ViSession instrumentHandle,
                                              ViPBoolean DCOffsetState);
ViStatus _VI_FUNC rssma_SetCSYNDCOffset (ViSession instrumentHandle,
                                         ViReal64 DCOffset);
ViStatus _VI_FUNC rssma_GetCSYNDCOffset (ViSession instrumentHandle,
                                         ViPReal64 DCOffset);
ViStatus _VI_FUNC rssma_confNoise (ViSession instrumentHandle,
                                   ViBoolean bandwidthState, ViReal64 bandwidth,
                                   ViInt32 distribution);
ViStatus _VI_FUNC rssma_SetNOISEBandwidthState (ViSession instrumentHandle,
                                                ViBoolean bandwidthState);
ViStatus _VI_FUNC rssma_GetNOISEBandwidthState (ViSession instrumentHandle,
                                                ViPBoolean bandwidthState);
ViStatus _VI_FUNC rssma_SetNOISEBandwidth (ViSession instrumentHandle,
                                           ViReal64 bandwidth);
ViStatus _VI_FUNC rssma_GetNOISEBandwidth (ViSession instrumentHandle,
                                           ViPReal64 bandwidth);
ViStatus _VI_FUNC rssma_SetNOISEDistribution (ViSession instrumentHandle,
                                              ViInt32 distribution);
ViStatus _VI_FUNC rssma_GetNOISEDistribution (ViSession instrumentHandle,
                                              ViPInt32 distribution);
ViStatus _VI_FUNC rssma_GetNOISERelativeLevel (ViSession instrumentHandle,
                                               ViPReal64 relativeLevel);
ViStatus _VI_FUNC rssma_GetNOISEAbsoluteLevel (ViSession instrumentHandle,
                                               ViPReal64 absoluteLevel);
ViStatus _VI_FUNC rssma_OvervoltageProtectionClear (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_OvervoltageProtectionTripped (ViSession instrumentHandle,
                                                      ViPInt32 protectiveCircuitState);
ViStatus _VI_FUNC rssma_CALibrationAll (ViSession instrumentHandle, ViInt32 timeout,
                                        ViPInt32 response);
ViStatus _VI_FUNC rssma_CALibrationClockSynthesis (ViSession instrumentHandle,
                                                   ViInt32 timeout,
                                                   ViPInt32 response);
ViStatus _VI_FUNC rssma_CALibrationFrequency (ViSession instrumentHandle,
                                              ViInt32 timeout, ViPInt32 response);
ViStatus _VI_FUNC rssma_CALibrationModulationGenerators (ViSession instrumentHandle,
                                                         ViInt32 timeout,
                                                         ViPInt32 response);
ViStatus _VI_FUNC rssma_CALibrationLevel (ViSession instrumentHandle,
                                          ViInt32 timeout, ViPInt32 response);
ViStatus _VI_FUNC rssma_SetCALibrationLevelState (ViSession instrumentHandle,
                                                  ViBoolean levelState);
ViStatus _VI_FUNC rssma_GetCALibrationLevelState (ViSession instrumentHandle,
                                                  ViPBoolean levelState);
ViStatus _VI_FUNC rssma_SetCALibrationLevelAdjustmentData
             (ViSession instrumentHandle, ViInt32 adjustmentData);
ViStatus _VI_FUNC rssma_GetCALibrationLevelAdjustmentData
             (ViSession instrumentHandle, ViPInt32 adjustmentData);
ViStatus _VI_FUNC rssma_CALibrationFMOffset (ViSession instrumentHandle,
                                             ViInt32 timeout, ViPInt32 response);
ViStatus _VI_FUNC rssma_CALibrationDMEMPulseSlope (ViSession instrumentHandle,
                                                   ViInt32 timeout,
                                                   ViPInt32 response);
ViStatus _VI_FUNC rssma_SetPULMExtInpImpedance (ViSession instrumentHandle,
                                                ViInt32 PULMExternalInputImpedance);
ViStatus _VI_FUNC rssma_GetPULMExtInpImpedance (ViSession instrumentHandle,
                                                ViPInt32 PULMExternalInputImpedance);
ViStatus _VI_FUNC rssma_SetPulseExtInpImpedance (ViSession instrumentHandle,
                                                 ViInt32 pulseExternalInputImpedance);
ViStatus _VI_FUNC rssma_GetPulseExtInpImpedance (ViSession instrumentHandle,
                                                 ViPInt32 pulseExternalInputImpedance);
ViStatus _VI_FUNC rssma_SetImageFileName (ViSession instrumentHandle,
                                          ViString fileName);
ViStatus _VI_FUNC rssma_GetImageFileName (ViSession instrumentHandle,
                                          ViChar _VI_FAR fileName[]);
ViStatus _VI_FUNC rssma_SetImageFormat (ViSession instrumentHandle,
                                        ViInt32 imageFormat);
ViStatus _VI_FUNC rssma_GetImageFormat (ViSession instrumentHandle,
                                        ViPInt32 imageFormat);
ViStatus _VI_FUNC rssma_GetImageSnapshot (ViSession instrumentHandle,
                                          ViString destination);
ViStatus _VI_FUNC rssma_SetImageSize (ViSession instrumentHandle,
                                      ViInt32 imageSize);
ViStatus _VI_FUNC rssma_GetImageSize (ViSession instrumentHandle,
                                      ViPInt32 imageSize);
ViStatus _VI_FUNC rssma_SetOutputDevice (ViSession instrumentHandle,
                                         ViInt32 outputDevice);
ViStatus _VI_FUNC rssma_GetOutputDevice (ViSession instrumentHandle,
                                         ViPInt32 outputDevice);
ViStatus _VI_FUNC rssma_SaveHardCopy (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetAutomaticNamingState (ViSession instrumentHandle,
                                                 ViBoolean state);
ViStatus _VI_FUNC rssma_GetAutomaticNamingState (ViSession instrumentHandle,
                                                 ViPBoolean state);
ViStatus _VI_FUNC rssma_GetAutomaticFilePath (ViSession instrumentHandle,
                                              ViInt32 arraySize,
                                              ViChar _VI_FAR filePath[]);
ViStatus _VI_FUNC rssma_GetAutomaticFileName (ViSession instrumentHandle,
                                              ViChar _VI_FAR fileName[]);
ViStatus _VI_FUNC rssma_SetAutomaticDirectory (ViSession instrumentHandle,
                                               ViString directoryName);
ViStatus _VI_FUNC rssma_GetAutomaticDirectory (ViSession instrumentHandle,
                                               ViChar _VI_FAR directory[]);
ViStatus _VI_FUNC rssma_DeleteImageFiles (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetAutomaticFileDateState (ViSession instrumentHandle,
                                                   ViInt32 fileNamePart,
                                                   ViBoolean state);
ViStatus _VI_FUNC rssma_GetAutomaticFileDateState (ViSession instrumentHandle,
                                                   ViInt32 fileNamePart,
                                                   ViPBoolean state);
ViStatus _VI_FUNC rssma_GetAutomaticFileDate (ViSession instrumentHandle,
                                              ViInt32 fileNamePart,
                                              ViChar _VI_FAR part[]);
ViStatus _VI_FUNC rssma_SetAutomaticFilePrefixState (ViSession instrumentHandle,
                                                     ViBoolean state);
ViStatus _VI_FUNC rssma_GetAutomaticFilePrefixState (ViSession instrumentHandle,
                                                     ViPBoolean state);
ViStatus _VI_FUNC rssma_SetAutomaticFilePrefix (ViSession instrumentHandle,
                                                ViString prefix);
ViStatus _VI_FUNC rssma_GetAutomaticFilePrefix (ViSession instrumentHandle,
                                                ViChar _VI_FAR fileName[]);
ViStatus _VI_FUNC rssma_GetAutomaticFileNumber (ViSession instrumentHandle,
                                                ViPInt32 number);
ViStatus _VI_FUNC rssma_ClearStatus (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_EventStatusEnable (ViSession instrumentHandle,
                                           ViInt32 registerValue);
ViStatus _VI_FUNC rssma_EventStatusQuery (ViSession instrumentHandle,
                                          ViPInt32 registerValue);
ViStatus _VI_FUNC rssma_IdentificationQuery (ViSession instrumentHandle,
                                             ViChar _VI_FAR instrumentIdentification[]);
ViStatus _VI_FUNC rssma_IndividualStatusQuery (ViSession instrumentHandle,
                                               ViPInt32 ISTFlag);
ViStatus _VI_FUNC rssma_OperationComplete (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_OperationCompleteQuery (ViSession instrumentHandle,
                                                ViPInt32 OPCState);
ViStatus _VI_FUNC rssma_OptionIdentificationQuery (ViSession instrumentHandle,
                                                   ViChar _VI_FAR optionIdentification[]);
ViStatus _VI_FUNC rssma_ParallelPollRegisterEnable (ViSession instrumentHandle,
                                                    ViInt32 registerValue);
ViStatus _VI_FUNC rssma_ParallelPollRegisterQuery (ViSession instrumentHandle,
                                                   ViPInt32 registerValue);
ViStatus _VI_FUNC rssma_PowerOnStatusClear (ViSession instrumentHandle,
                                            ViInt32 registerValue);
ViStatus _VI_FUNC rssma_PowerOnStatusClearQuery (ViSession instrumentHandle,
                                                 ViPInt32 registerValue);
ViStatus _VI_FUNC rssma_Recall (ViSession instrumentHandle,
                                ViInt32 instrumentStatus);
ViStatus _VI_FUNC rssma_Save (ViSession instrumentHandle, ViInt32 instrumentStatus);
ViStatus _VI_FUNC rssma_ServiceRequestEnable (ViSession instrumentHandle,
                                              ViInt32 registerValue);
ViStatus _VI_FUNC rssma_ServiceRequestEnableQuery (ViSession instrumentHandle,
                                                   ViPInt32 registerValue);
ViStatus _VI_FUNC rssma_StatusByteQuery (ViSession instrumentHandle,
                                         ViPInt32 statusByte);
ViStatus _VI_FUNC rssma_Trigger (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_TriggerWaitOPC (ViSession instrumentHandle,
                                        ViInt32 timeout);
ViStatus _VI_FUNC rssma_SelfTestQuery (ViSession instrumentHandle,
                                       ViPInt32 errorCode);
ViStatus _VI_FUNC rssma_WaittoContinue (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_PresetSignalPath (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_DIAGnosticModules (ViSession instrumentHandle,
                                           ViString moduleName, ViInt32 arraySize,
                                           ViChar _VI_FAR moduleConfiguration[]);
ViStatus _VI_FUNC rssma_DIAGnosticModuleNames (ViSession instrumentHandle,
                                               ViInt32 arraySize,
                                               ViChar _VI_FAR moduleNames[]);
ViStatus _VI_FUNC rssma_DIAGnosticTestPoints (ViSession instrumentHandle,
                                              ViInt32 arraySize,
                                              ViChar _VI_FAR testPoints[]);
ViStatus _VI_FUNC rssma_DIAGnosticTestPointVoltages (ViSession instrumentHandle,
                                                     ViString testPointName,
                                                     ViPReal64 measuredVoltage);
ViStatus _VI_FUNC rssma_DIAGnosticOperationTime (ViSession instrumentHandle,
                                                 ViPUInt32 operationTime);
ViStatus _VI_FUNC rssma_DIAGnosticPowerOnEvents (ViSession instrumentHandle,
                                                 ViPUInt32 powerOnEvents);
ViStatus _VI_FUNC rssma_TESTDirect (ViSession instrumentHandle, ViInt32 command,
                                    ViChar _VI_FAR assemblyResponse[]);
ViStatus _VI_FUNC rssma_TESTAllStart (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_TESTAllResult (ViSession instrumentHandle,
                                       ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rssma_SetDisplayPowerSaveState (ViSession instrumentHandle,
                                                  ViBoolean state);
ViStatus _VI_FUNC rssma_GetDisplayPowerSaveState (ViSession instrumentHandle,
                                                  ViPBoolean state);
ViStatus _VI_FUNC rssma_SetDisplayPowerSaveHoldoff (ViSession instrumentHandle,
                                                    ViInt32 holdoff);
ViStatus _VI_FUNC rssma_GetDisplayPowerSaveHoldoff (ViSession instrumentHandle,
                                                    ViPInt32 holdoff);
ViStatus _VI_FUNC rssma_SetDisplayUpdateState (ViSession instrumentHandle,
                                               ViBoolean state);
ViStatus _VI_FUNC rssma_GetDisplayUpdateState (ViSession instrumentHandle,
                                               ViPBoolean state);

ViStatus _VI_FUNC rssma_diplayDialogOpen (ViSession instrumentHandle,
                                          ViString dialog);

ViStatus _VI_FUNC rssma_diplayDialogClose (ViSession instrumentHandle,
                                           ViString dialog);

ViStatus _VI_FUNC rssma_getDisplayDialogID (ViSession instrumentHandle,
                                            ViInt32 arraySize,
                                            ViChar dialogIDs[]);

ViStatus _VI_FUNC rssma_SetKeyboardLanguage (ViSession instrumentHandle,
                                             ViInt32 language);

ViStatus _VI_FUNC rssma_GetKeyboardLanguage (ViSession instrumentHandle,
                                             ViInt32 *language);

ViStatus _VI_FUNC rssma_GetKeyboardLayout (ViSession instrumentHandle,
                                           ViInt32 *layout);

ViStatus _VI_FUNC rssma_MMEMoryCatalog (ViSession instrumentHandle,
                                        ViString directoryPath, ViInt32 arraySize,
                                        ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_MMEMoryCatalogLength (ViSession instrumentHandle,
                                              ViString directoryPath,
                                              ViPInt32 catalogLength);
ViStatus _VI_FUNC rssma_MMEMoryChangeDirectory (ViSession instrumentHandle,
                                                ViString directoryName);
ViStatus _VI_FUNC rssma_MMEMoryCopy (ViSession instrumentHandle,
                                     ViString sourcePath, ViString destinationPath);
ViStatus _VI_FUNC rssma_MMEMoryBlockDataWrite (ViSession instrumentHandle,
                                               ViString fileName,
                                               ViInt32 binaryBlockLength,
                                               ViChar _VI_FAR binaryBlockData[]);
ViStatus _VI_FUNC rssma_MMEMoryBlockDataRead (ViSession instrumentHandle,
                                              ViString fileName,
                                              ViInt32 binaryBlockLength,
                                              ViChar _VI_FAR binaryBlockData[]);
ViStatus _VI_FUNC rssma_MMEMoryDirectoryCatalog (ViSession instrumentHandle,
                                                 ViString directoryPath,
                                                 ViInt32 arraySize,
                                                 ViChar _VI_FAR catalogList[]);
ViStatus _VI_FUNC rssma_MMEMoryDirectoryCatalogLength (ViSession instrumentHandle,
                                                       ViString directoryPath,
                                                       ViPInt32 catalogLength);
ViStatus _VI_FUNC rssma_MMEMoryDelete (ViSession instrumentHandle,
                                       ViString fileName);
ViStatus _VI_FUNC rssma_MMEMoryLoad (ViSession instrumentHandle,
                                     ViInt32 instrumentSetting, ViString fileName);
ViStatus _VI_FUNC rssma_MMEMoryMakeDirectory (ViSession instrumentHandle,
                                              ViString directoryName);
ViStatus _VI_FUNC rssma_MMEMoryMove (ViSession instrumentHandle,
                                     ViString fileSource, ViString fileDestination);
ViStatus _VI_FUNC rssma_MMEMoryRemoveDirectory (ViSession instrumentHandle,
                                                ViString directoryName);
ViStatus _VI_FUNC rssma_MMEMoryStore (ViSession instrumentHandle,
                                      ViInt32 instrumentSetting, ViString fileName);
ViStatus _VI_FUNC rssma_setRegister (ViSession instrumentHandle,
                                     ViInt32 registerGroup, ViInt32 registerType,
                                     ViInt32 value);
ViStatus _VI_FUNC rssma_getRegister (ViSession instrumentHandle,
                                     ViInt32 registerGroup, ViInt32 registerType,
                                     ViPInt32 value);
ViStatus _VI_FUNC rssma_STATusPRESet (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetSYSTemBeeper (ViSession instrumentHandle,
                                         ViBoolean systemBeeper);
ViStatus _VI_FUNC rssma_GetSYSTemBeeper (ViSession instrumentHandle,
                                         ViPBoolean systemBeeper);
ViStatus _VI_FUNC rssma_SYSTemErrorQueueAll (ViSession instrumentHandle,
                                             ViInt32 arraySize,
                                             ViChar _VI_FAR errorQueue[]);
ViStatus _VI_FUNC rssma_SYSTemErrorQueueCodeAll (ViSession instrumentHandle,
                                                 ViInt32 arraySize,
                                                 ViChar _VI_FAR errorQueueCode[]);
ViStatus _VI_FUNC rssma_SYSTemErrorQueueCodeNext (ViSession instrumentHandle,
                                                  ViPInt32 errorQueueCodeNext);
ViStatus _VI_FUNC rssma_SYSTemErrorQueueCount (ViSession instrumentHandle,
                                               ViPInt32 errorQueueCount);

ViStatus _VI_FUNC rssma_SYSTemErrorAll (ViSession instrumentHandle,
                                        ViInt32 arraySize, ViChar errorQueue[]);

ViStatus _VI_FUNC rssma_SYSTemUSBResource (ViSession instrumentHandle,
                                           ViInt32 arraySize,
                                           ViChar _VI_FAR USBResource[]);
ViStatus _VI_FUNC rssma_SYSTemGPIBResource (ViSession instrumentHandle,
                                            ViInt32 arraySize,
                                            ViChar _VI_FAR GPIBResource[]);
ViStatus _VI_FUNC rssma_SYSTemSerialResource (ViSession instrumentHandle,
                                              ViInt32 arraySize,
                                              ViChar _VI_FAR serialResource[]);

ViStatus _VI_FUNC rssma_SYSTemHISLIPResource (ViSession instrumentHandle,
                                              ViInt32 arraySize,
                                              ViChar hiSLIPResource[]);

ViStatus _VI_FUNC rssma_SYSTemSocketResource (ViSession instrumentHandle,
                                              ViInt32 arraySize,
                                              ViChar socketResource[]);

ViStatus _VI_FUNC rssma_SYSTemOptionUserDefined (ViSession instrumentHandle,
                                                 ViString userDefinedIdentification);

ViStatus _VI_FUNC rssma_SYSTemIdentificationUserDefined (ViSession instrumentHandle,
                                                         ViString userDefinedIdentification);

ViStatus _VI_FUNC rssma_SYSTemLanguage (ViSession instrumentHandle,
                                        ViInt32 language);

ViStatus _VI_FUNC rssma_SetSYSTemAnnotationAmplitude (ViSession instrumentHandle,
                                                      ViBoolean amplitude);

ViStatus _VI_FUNC rssma_GetSYSTemAnnotationAmplitude (ViSession instrumentHandle,
                                                      ViBoolean *amplitude);

ViStatus _VI_FUNC rssma_SetSystemAnnotationFrequency (ViSession instrumentHandle,
                                                      ViBoolean frequency);

ViStatus _VI_FUNC rssma_GetSystemAnnotationFrequency (ViSession instrumentHandle,
                                                      ViBoolean *frequency);

ViStatus _VI_FUNC rssma_SetSYSTemAnnotationAll (ViSession instrumentHandle,
                                                ViBoolean state);

ViStatus _VI_FUNC rssma_GetSYSTemAnnotationAll (ViSession instrumentHandle,
                                                ViBoolean *state);

ViStatus _VI_FUNC rssma_SetSYSTemKeyboardLock (ViSession instrumentHandle,
                                               ViBoolean keyboardLock);
ViStatus _VI_FUNC rssma_GetSYSTemKeyboardLock (ViSession instrumentHandle,
                                               ViPBoolean keyboardLock);
ViStatus _VI_FUNC rssma_SetSYSTemDisplayLock (ViSession instrumentHandle,
                                              ViBoolean displayLock);
ViStatus _VI_FUNC rssma_GetSYSTemDisplayLock (ViSession instrumentHandle,
                                              ViPBoolean displayLock);
ViStatus _VI_FUNC rssma_SetSYSTemProtect (ViSession instrumentHandle,
                                          ViInt32 protectionLevel,
                                          ViBoolean protection, ViString password);
ViStatus _VI_FUNC rssma_GetSYSTemVersion (ViSession instrumentHandle,
                                          ViChar _VI_FAR SCPIVersion[]);
ViStatus _VI_FUNC rssma_GetSYSTemBIOSVersion (ViSession instrumentHandle,
                                              ViChar _VI_FAR BIOSVersion[]);
ViStatus _VI_FUNC rssma_SetSYSTemIdentification (ViSession instrumentHandle,
                                                 ViInt32 identification);
ViStatus _VI_FUNC rssma_GetSYSTemIdentification (ViSession instrumentHandle,
                                                 ViPInt32 identification);
ViStatus _VI_FUNC rssma_GetSYSTemStartupStatus (ViSession instrumentHandle,
                                                ViPInt32 startupComplete);

ViStatus _VI_FUNC rssma_SetSYSTemDate (ViSession instrumentHandle, ViInt32 year,
                                       ViInt32 month, ViInt32 day);

ViStatus _VI_FUNC rssma_GetSYSTemDate (ViSession instrumentHandle, ViInt32 *year,
                                       ViInt32 *month, ViInt32 *day);

ViStatus _VI_FUNC rssma_SetSYSTemTime (ViSession instrumentHandle, ViInt32 hour,
                                       ViInt32 min, ViInt32 sec);

ViStatus _VI_FUNC rssma_GetSYSTemTime (ViSession instrumentHandle, ViInt32 *hour,
                                       ViInt32 *min, ViInt32 *sec);

ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddress (ViSession instrumentHandle,
                                                   ViString IPAddress);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddress (ViSession instrumentHandle,
                                                   ViInt32 arraySize,
                                                   ViChar _VI_FAR IPAddress[]);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddressSubnetMask
             (ViSession instrumentHandle, ViString IPAddressSubnetMask);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddressSubnetMask
             (ViSession instrumentHandle, ViInt32 arraySize,
              ViChar _VI_FAR IPAddressSubnetMask[]);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddressGateway (ViSession instrumentHandle,
                                                          ViString IPAddressGateway);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddressGateway (ViSession instrumentHandle,
                                                          ViInt32 arraySize,
                                                          ViChar _VI_FAR IPAddressGateway[]);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddressMode (ViSession instrumentHandle,
                                                       ViInt32 IPAddressMode);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddressMode (ViSession instrumentHandle,
                                                       ViPInt32 IPAddressMode);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkCommonDomain (ViSession instrumentHandle,
                                                      ViString commonDomain);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkCommonDomain (ViSession instrumentHandle,
                                                      ViInt32 arraySize,
                                                      ViChar _VI_FAR commonDomain[]);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkCommonHostname (ViSession instrumentHandle,
                                                        ViString commonHostname);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkCommonHostname (ViSession instrumentHandle,
                                                        ViInt32 arraySize,
                                                        ViChar _VI_FAR commonHostname[]);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkDomain (ViSession instrumentHandle,
                                                ViString domain);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkDomain (ViSession instrumentHandle,
                                                ViInt32 arraySize,
                                                ViChar _VI_FAR domain[]);

ViStatus _VI_FUNC rssma_SetSYSTemNetworkWorkgroup (ViSession instrumentHandle,
                                                   ViString workgroupName);

ViStatus _VI_FUNC rssma_GetSYSTemNetworkWorkgroup (ViSession instrumentHandle,
                                                   ViInt32 arraySize,
                                                   ViChar workgroupName[]);

ViStatus _VI_FUNC rssma_SetSYSTemNetworkGateway (ViSession instrumentHandle,
                                                 ViString gateway);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkGateway (ViSession instrumentHandle,
                                                 ViInt32 arraySize,
                                                 ViChar _VI_FAR gateway[]);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkResource (ViSession instrumentHandle,
                                                  ViInt32 arraySize,
                                                  ViChar _VI_FAR resource[]);
ViStatus _VI_FUNC rssma_SetSYSTemNetworkSubnetMask (ViSession instrumentHandle,
                                                    ViString subnetMask);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkSubnetMask (ViSession instrumentHandle,
                                                    ViInt32 arraySize,
                                                    ViChar _VI_FAR subnetMask[]);
ViStatus _VI_FUNC rssma_GetSYSTemNetworkMACAddress (ViSession instrumentHandle,
                                                    ViInt32 arraySize,
                                                    ViChar _VI_FAR MACAddress[]);
ViStatus _VI_FUNC rssma_GetSYSTemNetwork (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetInstrumentTriggerInputSlope (ViSession instrumentHandle,
                                                        ViInt32 inputTriggerSlope);
ViStatus _VI_FUNC rssma_GetInstrumentTriggerInputSlope (ViSession instrumentHandle,
                                                        ViPInt32 inputTriggerSlope);
ViStatus _VI_FUNC rssma_TriggerSourceAllSweeps (ViSession instrumentHandle,
                                                ViInt32 triggerSourceAllSweeps);
ViStatus _VI_FUNC rssma_TriggerAllSweeps (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_TriggerRFFrequencySweep (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_TriggerRFLevelSweep (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_TriggerListMode (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_SetAngleUnits (ViSession instrumentHandle,
                                       ViInt32 angleUnits);
ViStatus _VI_FUNC rssma_GetAngleUnits (ViSession instrumentHandle,
                                       ViPInt32 angleUnits);
ViStatus _VI_FUNC rssma_SetPowerUnits (ViSession instrumentHandle,
                                       ViInt32 powerUnits);
ViStatus _VI_FUNC rssma_GetPowerUnits (ViSession instrumentHandle,
                                       ViPInt32 powerUnits);
ViStatus _VI_FUNC rssma_SetVelocityUnits (ViSession instrumentHandle,
                                          ViInt32 velocityUnits);
ViStatus _VI_FUNC rssma_GetVelocityUnits (ViSession instrumentHandle,
                                          ViPInt32 velocityUnits);
ViStatus _VI_FUNC rssma_readToFile (ViSession instrumentHandle, ViString source,
                                    ViString destination);
ViStatus _VI_FUNC rssma_writeFromFile (ViSession instrumentHandle, ViString source,
                                       ViString destination);
ViStatus _VI_FUNC rssma_errorCheckState (ViSession instrumentHandle,
                                         ViBoolean stateChecking);
ViStatus _VI_FUNC rssma_optionCheckState (ViSession instrumentHandle,
                                          ViBoolean stateChecking);
ViStatus _VI_FUNC rssma_rangeCheckState (ViSession instrumentHandle,
                                         ViBoolean rangeChecking);
ViStatus _VI_FUNC rssma_writeInstrData (ViSession instrumentHandle,
                                        ViString writeBuffer);
ViStatus _VI_FUNC rssma_readInstrData (ViSession instrumentHandle,
                                       ViInt32 numberBytesToRead,
                                       ViChar _VI_FAR readBuffer[],
                                       ViPInt32 numBytesRead);
ViStatus _VI_FUNC rssma_DevicePreset (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rssma_self_test (ViSession instrumentHandle,
                                   ViPInt16 selfTestResult,
                                   ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rssma_error_query (ViSession instrumentHandle, ViPInt32 errorCode,
                                     ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rssma_error_message (ViSession instrumentHandle,
                                       ViStatus statusCode,
                                       ViChar _VI_FAR message[]);
ViStatus _VI_FUNC rssma_revision_query (ViSession instrumentHandle,
                                        ViChar _VI_FAR instrumentDriverRevision[],
                                        ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rssma_delay (ViSession instrumentHandle, ViReal64 seconds);
ViStatus _VI_FUNC rssma_close (ViSession instrumentHandle);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
